@@ -1,5 +1,5 @@
 /*
-    $Id: aspi32.c,v 1.9 2004/06/25 01:47:06 rocky Exp $
+    $Id: aspi32.c,v 1.49 2004/08/27 04:12:29 rocky Exp $
 
     Copyright (C) 2004 Rocky Bernstein <rocky@panix.com>
 
@@ -23,11 +23,11 @@
    Inspired by vlc's cdrom.h code 
 */
 
-#ifdef HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H 
 # include "config.h"
 #endif
 
-static const char _rcsid[] = "$Id: aspi32.c,v 1.9 2004/06/25 01:47:06 rocky Exp $";
+static const char _rcsid[] = "$Id: aspi32.c,v 1.49 2004/08/27 04:12:29 rocky Exp $";
 
 #include <cdio/cdio.h>
 #include <cdio/sector.h>
@@ -52,6 +52,82 @@ static const char _rcsid[] = "$Id: aspi32.c,v 1.9 2004/06/25 01:47:06 rocky Exp
 #include <sys/stat.h>
 #include <sys/types.h>
 #include "aspi32.h"
+#include "cdtext_private.h"
+
+/* Amount of time we are willing to wait for an operation to complete.
+   10 seconds? 
+*/
+#define OP_TIMEOUT_MS 10000
+
+static const 
+char *aspierror(int nErrorCode) 
+{
+  switch (nErrorCode)
+    {
+    case SS_PENDING:
+      return "SRB being processed";
+      break;
+    case SS_COMP:
+      return "SRB completed without error";
+      break;
+    case SS_ABORTED:
+      return "SRB aborted";
+      break;
+    case SS_ABORT_FAIL:
+      return "Unable to abort SRB";
+      break;
+    case SS_ERR:
+      return "SRB completed with error";
+      break;
+    case SS_INVALID_CMD:
+      return "Invalid ASPI command";
+      break;
+    case SS_INVALID_HA:
+      return "Invalid host adapter number";
+      break;
+    case SS_NO_DEVICE:
+      return "SCSI device not installed";
+      break;
+    case SS_INVALID_SRB:
+      return "Invalid parameter set in SRB";
+      break;
+    case SS_OLD_MANAGER:
+      return "ASPI manager doesn't support";
+      break;
+    case SS_ILLEGAL_MODE:
+      return "Unsupported MS Windows mode";
+      break;
+    case SS_NO_ASPI:
+      return "No ASPI managers";
+      break;
+    case SS_FAILED_INIT:
+      return "ASPI for windows failed init";
+      break;
+    case SS_ASPI_IS_BUSY:
+      return "No resources available to execute command.";
+      break;
+    case SS_BUFFER_TOO_BIG:
+      return "Buffer size is too big to handle.";
+      break;
+    case SS_MISMATCHED_COMPONENTS:
+      return "The DLLs/EXEs of ASPI don't version check";
+      break;
+    case SS_NO_ADAPTERS:
+      return "No host adapters found";
+      break;
+    case SS_INSUFFICIENT_RESOURCES:
+      return "Couldn't allocate resources needed to init";
+      break;
+    case SS_ASPI_IS_SHUTDOWN:
+      return "Call came to ASPI after PROCESS_DETACH";
+      break;
+    case SS_BAD_INSTALL:
+      return "The DLL or other components are installed wrong.";
+      break;
+    default: 
+      return "Unknown ASPI error.";
+    }
+}
 
 /* General ioctl() CD-ROM command function */
 static bool 
@@ -101,6 +177,96 @@ have_aspi( HMODULE *hASPI,
   return true;
 }
 
+/*! 
+  Get disc type associated with cd object.
+*/
+discmode_t
+get_discmode_aspi (_img_private_t *p_env)
+{
+  track_t i_track;
+  discmode_t discmode=CDIO_DISC_MODE_NO_INFO;
+
+  /* See if this is a DVD. */
+  cdio_dvd_struct_t dvd;  /* DVD READ STRUCT for layer 0. */
+
+  dvd.physical.type = CDIO_DVD_STRUCT_PHYSICAL;
+  dvd.physical.layer_num = 0;
+  if (0 == scsi_mmc_get_dvd_struct_physical_private (p_env, 
+						     &run_scsi_cmd_aspi,
+						     &dvd)) {
+    switch(dvd.physical.layer[0].book_type) {
+    case CDIO_DVD_BOOK_DVD_ROM:  return CDIO_DISC_MODE_DVD_ROM;
+    case CDIO_DVD_BOOK_DVD_RAM:  return CDIO_DISC_MODE_DVD_RAM;
+    case CDIO_DVD_BOOK_DVD_R:    return CDIO_DISC_MODE_DVD_R;
+    case CDIO_DVD_BOOK_DVD_RW:   return CDIO_DISC_MODE_DVD_RW;
+    case CDIO_DVD_BOOK_DVD_PR:   return CDIO_DISC_MODE_DVD_PR;
+    case CDIO_DVD_BOOK_DVD_PRW:  return CDIO_DISC_MODE_DVD_PRW;
+    default: return CDIO_DISC_MODE_DVD_OTHER;
+    }
+  }
+
+  if (!p_env->gen.toc_init) 
+    read_toc_aspi (p_env);
+
+  if (!p_env->gen.toc_init) 
+    return CDIO_DISC_MODE_NO_INFO;
+
+  for (i_track = p_env->gen.i_first_track; 
+       i_track < p_env->gen.i_first_track + p_env->gen.i_tracks ; 
+       i_track ++) {
+    track_format_t track_fmt=get_track_format_aspi(p_env, i_track);
+
+    switch(track_fmt) {
+    case TRACK_FORMAT_AUDIO:
+      switch(discmode) {
+	case CDIO_DISC_MODE_NO_INFO:
+	  discmode = CDIO_DISC_MODE_CD_DA;
+	  break;
+	case CDIO_DISC_MODE_CD_DA:
+	case CDIO_DISC_MODE_CD_MIXED: 
+	case CDIO_DISC_MODE_ERROR: 
+	  /* No change*/
+	  break;
+      default:
+	  discmode = CDIO_DISC_MODE_CD_MIXED;
+      }
+      break;
+    case TRACK_FORMAT_XA:
+      switch(discmode) {
+	case CDIO_DISC_MODE_NO_INFO:
+	  discmode = CDIO_DISC_MODE_CD_XA;
+	  break;
+	case CDIO_DISC_MODE_CD_XA:
+	case CDIO_DISC_MODE_CD_MIXED: 
+	case CDIO_DISC_MODE_ERROR: 
+	  /* No change*/
+	  break;
+      default:
+	discmode = CDIO_DISC_MODE_CD_MIXED;
+      }
+      break;
+    case TRACK_FORMAT_DATA:
+      switch(discmode) {
+	case CDIO_DISC_MODE_NO_INFO:
+	  discmode = CDIO_DISC_MODE_CD_DATA;
+	  break;
+	case CDIO_DISC_MODE_CD_DATA:
+	case CDIO_DISC_MODE_CD_MIXED: 
+	case CDIO_DISC_MODE_ERROR: 
+	  /* No change*/
+	  break;
+      default:
+	discmode = CDIO_DISC_MODE_CD_MIXED;
+      }
+      break;
+    case TRACK_FORMAT_ERROR:
+    default:
+      discmode = CDIO_DISC_MODE_ERROR;
+    }
+  }
+  return discmode;
+}
+
 const char *
 is_cdrom_aspi(const char drive_letter) 
 {
@@ -111,6 +277,7 @@ is_cdrom_aspi(const char drive_letter)
   DWORD dwSupportInfo;
   int i_adapter, i_hostadapters;
   char c_drive;
+  int i_rc;
 
   if ( !have_aspi(&hASPI, &lpGetSupport, &lpSendCommand) )
     return NULL;
@@ -118,15 +285,11 @@ is_cdrom_aspi(const char drive_letter)
   /* ASPI support seems to be there. */
   
   dwSupportInfo = lpGetSupport();
-  
-  if( HIBYTE( LOWORD ( dwSupportInfo ) ) == SS_NO_ADAPTERS ) {
-    cdio_debug("no host adapters found (ASPI)");
-    FreeLibrary( hASPI );
-    return NULL;
-  }
-  
-  if( HIBYTE( LOWORD ( dwSupportInfo ) ) != SS_COMP ) {
-    cdio_debug("Unable to initalize ASPI layer");
+
+  i_rc = HIBYTE( LOWORD ( dwSupportInfo ) );
+
+  if( SS_COMP != i_rc ) {
+    cdio_debug("ASPI: %s", aspierror(i_rc));
     FreeLibrary( hASPI );
     return NULL;
   }
@@ -205,6 +368,7 @@ init_aspi (_img_private_t *env)
   DWORD dwSupportInfo;
   int i_adapter, i_hostadapters;
   char c_drive;
+  int i_rc;
 
   if (2 == strlen(env->gen.source_name) && isalpha(env->gen.source_name[0]) ) 
   {
@@ -212,6 +376,8 @@ init_aspi (_img_private_t *env)
   } else if ( 6 == strlen(env->gen.source_name) 
 	      && isalpha(env->gen.source_name[4] )) {
     c_drive = env->gen.source_name[4];
+  } else {
+    c_drive = 'C';
   }
   
   if ( !have_aspi(&hASPI, &lpGetSupport, &lpSendCommand) )
@@ -221,14 +387,10 @@ init_aspi (_img_private_t *env)
   
   dwSupportInfo = lpGetSupport();
   
-  if( HIBYTE( LOWORD ( dwSupportInfo ) ) == SS_NO_ADAPTERS ) {
-    cdio_debug("no host adapters found (ASPI)");
-    FreeLibrary( hASPI );
-    return false;
-  }
-  
-  if( HIBYTE( LOWORD ( dwSupportInfo ) ) != SS_COMP ) {
-    cdio_debug("unable to initalize ASPI layer");
+  i_rc = HIBYTE( LOWORD ( dwSupportInfo ) );
+
+  if( SS_COMP != i_rc ) {
+    cdio_info("ASPI: %s", aspierror(i_rc));
     FreeLibrary( hASPI );
     return false;
   }
@@ -311,101 +473,138 @@ init_aspi (_img_private_t *env)
 }
 
 /*!
-   Reads a single mode2 sector from cd device into data starting from lsn.
-   Returns 0 if no error. 
+  Run a SCSI MMC command. 
+ 
+  env	        private CD structure 
+  i_timeout_ms  time in milliseconds we will wait for the command
+                to complete. If this value is -1, use the default 
+		time-out value.
+  p_buf	        Buffer for data, both sending and receiving
+  i_buf	        Size of buffer
+  e_direction	direction the transfer is to go.
+  cdb	        CDB bytes. All values that are needed should be set on 
+                input. We'll figure out what the right CDB length should be.
+
+  We return 0 if command completed successfully.
  */
-static int
-mmc_read_sectors_aspi (const _img_private_t *env, void *data, lsn_t lsn, 
-		       int sector_type, unsigned int nblocks)
+int
+run_scsi_cmd_aspi( const void *p_user_data, unsigned int i_timeout_ms,
+		   unsigned int i_cdb, const scsi_mmc_cdb_t * p_cdb,  
+		   scsi_mmc_direction_t e_direction, 
+		   unsigned int i_buf, /*in/out*/ void *p_buf )
 {
+  const _img_private_t *p_env = p_user_data;
   HANDLE hEvent;
   struct SRB_ExecSCSICmd ssc;
 
-#if 0  
-  sector_type = 0; /*all types */
-#endif
-  int sync        = 0;
-  int header_code = 2;
-  int i_user_data   = 1;
-  int edc_ecc     = 0;
-  int error_field = 0;
-  
-  
   /* Create the transfer completion event */
   hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
   if( hEvent == NULL ) {
+    cdio_info("CreateEvent failed");
     return 1;
   }
   
-  /* Data selection */
-  
   memset( &ssc, 0, sizeof( ssc ) );
-  
+
   ssc.SRB_Cmd         = SC_EXEC_SCSI_CMD;
-  ssc.SRB_Flags       = SRB_DIR_IN | SRB_EVENT_NOTIFY;
-  ssc.SRB_HaId        = LOBYTE( env->i_sid );
-  ssc.SRB_Target      = HIBYTE( env->i_sid );
-  ssc.SRB_Lun         = env->i_lun;
+
+  ssc.SRB_Flags       = SCSI_MMC_DATA_READ == e_direction ? 
+    SRB_DIR_IN | SRB_EVENT_NOTIFY : SRB_DIR_OUT | SRB_EVENT_NOTIFY;
+
+  ssc.SRB_HaId        = LOBYTE( p_env->i_sid );
+  ssc.SRB_Target      = HIBYTE( p_env->i_sid );
+  ssc.SRB_Lun         = p_env->i_lun;
   ssc.SRB_SenseLen    = SENSE_LEN;
   
   ssc.SRB_PostProc = (LPVOID) hEvent;
-  ssc.SRB_CDBLen      = 12;
+  ssc.SRB_CDBLen      = i_cdb;
   
-  CDIO_MMC_SET_COMMAND(ssc.CDBByte, CDIO_MMC_GPCMD_READ_CD);
-  CDIO_MMC_SET_READ_TYPE(ssc.CDBByte, sector_type);
-  CDIO_MMC_SET_READ_LBA(ssc.CDBByte, lsn);
-  CDIO_MMC_SET_READ_LENGTH(ssc.CDBByte, nblocks);
+  /* Result buffer */
+  ssc.SRB_BufPointer  = p_buf;
+  ssc.SRB_BufLen      = i_buf;
+
+  memcpy( ssc.CDBByte, p_cdb, i_cdb );
+  
+  ResetEvent( hEvent );
+  p_env->lpSendCommand( (void*) &ssc );
+  
+  /* If the command has still not been processed, wait until it's
+   * finished */
+  if( ssc.SRB_Status == SS_PENDING ) {
+    WaitForSingleObject( hEvent, msecs2secs(i_timeout_ms) );
+  }
+  CloseHandle( hEvent );
+  
+  /* check that the transfer went as planned */
+  if( ssc.SRB_Status != SS_COMP ) {
+    cdio_info("ASPI: %s", aspierror(ssc.SRB_Status));
+    return 2;
+  }
+
+  return 0;
+}
+
+
+/*!
+   Reads nblocks sectors from cd device into data starting from lsn.
+   Returns 0 if no error. 
+ */
+static int
+read_sectors_aspi (const _img_private_t *env, void *data, lsn_t lsn, 
+		   int sector_type, unsigned int nblocks)
+{
+  scsi_mmc_cdb_t cdb = {{0, }};
+  unsigned int i_buf;
+
+  int sync        = 0;
+  int header_code = 2;
+  int i_user_data = 1;
+  int edc_ecc     = 0;
+  int error_field = 0;
+  
+#if 0  
+  sector_type = 0; /*all types */
+#endif
+  
+  /* Set up passthrough command */
+  CDIO_MMC_SET_COMMAND      (cdb.field, CDIO_MMC_GPCMD_READ_CD);
+  CDIO_MMC_SET_READ_TYPE    (cdb.field, sector_type);
+  CDIO_MMC_SET_READ_LBA     (cdb.field, lsn);
+  CDIO_MMC_SET_READ_LENGTH24(cdb.field, nblocks);
   
 #if 1
-  ssc.CDBByte[ 9 ] = (sync << 7) |
+  cdb.field[ 9 ] = (sync << 7) |
     (header_code << 5) |
     (i_user_data << 4) |
     (edc_ecc << 3) |
     (error_field << 1);
   /* ssc.CDBByte[ 9 ] = READ_CD_USERDATA_MODE2; */
 #else 
-  CDIO_MMC_SET_MAIN_CHANNEL_SELECTION_BITS(ssc.CDBByte,
+  CDIO_MMC_SET_MAIN_CHANNEL_SELECTION_BITS(cmd,
 					   CDIO_MMC_MCSB_ALL_HEADERS);
 #endif
   
-  /* Result buffer */
-  ssc.SRB_BufPointer  = data;
-
   switch (sector_type) {
   case CDIO_MMC_READ_TYPE_ANY: 
   case CDIO_MMC_READ_TYPE_CDDA: 
-    ssc.SRB_BufLen = CDIO_CD_FRAMESIZE_RAW;
+    i_buf = CDIO_CD_FRAMESIZE_RAW;
     break;
   case CDIO_MMC_READ_TYPE_M2F1:
-    ssc.SRB_BufLen = CDIO_CD_FRAMESIZE;
+    i_buf = CDIO_CD_FRAMESIZE;
     break;
   case CDIO_MMC_READ_TYPE_M2F2:
-    ssc.SRB_BufLen = 2324;
+    i_buf = 2324;
     break;
   case CDIO_MMC_READ_TYPE_MODE1:
-    ssc.SRB_BufLen = CDIO_CD_FRAMESIZE;
+    i_buf = CDIO_CD_FRAMESIZE;
     break;
   default:
-    ssc.SRB_BufLen = CDIO_CD_FRAMESIZE_RAW;
-  }
-  
-  /* Initiate transfer */
-  ResetEvent( hEvent );
-  env->lpSendCommand( (void*) &ssc );
-  
-  /* If the command has still not been processed, wait until it's
-   * finished */
-  if( ssc.SRB_Status == SS_PENDING ) {
-    WaitForSingleObject( hEvent, INFINITE );
-  }
-  CloseHandle( hEvent );
-  
-  /* check that the transfer went as planned */
-  if( ssc.SRB_Status != SS_COMP ) {
-    return 1;
+    i_buf = CDIO_CD_FRAMESIZE_RAW;
   }
 
-  return 0;
+  return run_scsi_cmd_aspi(env, OP_TIMEOUT_MS, 
+			       scsi_mmc_get_cmd_len(cdb.field[0]), 
+			       &cdb, SCSI_MMC_DATA_READ, i_buf*nblocks, data);
 }
 
 /*!
@@ -416,8 +615,8 @@ int
 read_audio_sectors_aspi (_img_private_t *env, void *data, lsn_t lsn, 
 			     unsigned int nblocks) 
 {
-  if (mmc_read_sectors_aspi(env, data, lsn, CDIO_MMC_READ_TYPE_CDDA, 1)) {
-    return mmc_read_sectors_aspi(env, data, lsn, CDIO_MMC_READ_TYPE_ANY, 1);
+  if (read_sectors_aspi(env, data, lsn, CDIO_MMC_READ_TYPE_CDDA, 1)) {
+    return read_sectors_aspi(env, data, lsn, CDIO_MMC_READ_TYPE_ANY, 1);
   }
   return 0;
 }
@@ -430,7 +629,7 @@ int
 read_mode2_sector_aspi (const _img_private_t *env, void *data, lsn_t lsn, 
 			 bool b_form2)
 {
-  return mmc_read_sectors_aspi(env, data, lsn, b_form2 
+  return read_sectors_aspi(env, data, lsn, b_form2 
 			       ? CDIO_MMC_READ_TYPE_M2F2 
 			       : CDIO_MMC_READ_TYPE_M2F1, 
 			       1);
@@ -444,7 +643,7 @@ int
 read_mode1_sector_aspi (const _img_private_t *env, void *data, lsn_t lsn, 
 			 bool b_form2)
 {
-  return mmc_read_sectors_aspi(env, data, lsn, CDIO_MMC_READ_TYPE_MODE1, 1);
+  return read_sectors_aspi(env, data, lsn, CDIO_MMC_READ_TYPE_MODE1, 1);
 }
 
 /*! 
@@ -452,116 +651,73 @@ read_mode1_sector_aspi (const _img_private_t *env, void *data, lsn_t lsn,
   Return true if successful or false if an error.
 */
 bool
-read_toc_aspi (_img_private_t *env) 
+read_toc_aspi (_img_private_t *p_env) 
 {
-  HANDLE hEvent;
-  struct SRB_ExecSCSICmd ssc;
-  unsigned char p_tocheader[ 4 ];
-  
-  /* Create the transfer completion event */
-  hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
-  if( hEvent == NULL ) {
-    return false;
-  }
-  
-  memset( &ssc, 0, sizeof( ssc ) );
-  
-  ssc.SRB_Cmd         = SC_EXEC_SCSI_CMD;
-  ssc.SRB_Flags       = SRB_DIR_IN | SRB_EVENT_NOTIFY;
-  ssc.SRB_HaId        = LOBYTE( env->i_sid );
-  ssc.SRB_Target      = HIBYTE( env->i_sid );
-  ssc.SRB_Lun         = env->i_lun;
-  ssc.SRB_SenseLen    = SENSE_LEN;
-  
-  ssc.SRB_PostProc = (LPVOID) hEvent;
-  ssc.SRB_CDBLen      = 10;
+  scsi_mmc_cdb_t cdb = {{0, }};
+  unsigned char tocheader[ 4 ];
+  int      i_status;
   
   /* Operation code */
-  CDIO_MMC_SET_COMMAND(ssc.CDBByte, CDIO_MMC_READ_TOC);
+  CDIO_MMC_SET_COMMAND(cdb.field, CDIO_MMC_GPCMD_READ_TOC);
   
   /* Format */
-  ssc.CDBByte[ 2 ] = READ_TOC_FORMAT_TOC;
+  cdb.field[ 2 ] = CDIO_MMC_READTOC_FMT_TOC;
   
   /* Starting track */
-  CDIO_MMC_SET_START_TRACK(ssc.CDBByte, 0);
+  CDIO_MMC_SET_START_TRACK(cdb.field, 0);
   
-  /* Allocation length and buffer */
-  ssc.SRB_BufLen = sizeof( p_tocheader );
-  ssc.SRB_BufPointer  = p_tocheader;
-  ssc.CDBByte[ 7 ] = (unsigned char) ( ssc.SRB_BufLen >>  8 ) & 0xff;
-  ssc.CDBByte[ 8 ] = (unsigned char) ( ssc.SRB_BufLen       ) & 0xff;
+  CDIO_MMC_SET_READ_LENGTH16(cdb.field, sizeof(tocheader));
+
+  i_status = run_scsi_cmd_aspi (p_env, OP_TIMEOUT_MS,
+				    scsi_mmc_get_cmd_len(cdb.field[0]), 
+				    &cdb, SCSI_MMC_DATA_READ, 
+				    sizeof(tocheader), &tocheader);
   
-  /* Initiate transfer */
-  ResetEvent( hEvent );
-  env->lpSendCommand( (void*) &ssc );
+  if (0 != i_status) return false;
   
-  /* If the command has still not been processed, wait until it's
-   * finished */
-  if( ssc.SRB_Status == SS_PENDING )
-    WaitForSingleObject( hEvent, INFINITE );
-  
-  /* check that the transfer went as planned */
-  if( ssc.SRB_Status != SS_COMP ) {
-    CloseHandle( hEvent );
-    return false;
-  }
-  
-  env->i_first_track = p_tocheader[2];
-  env->total_tracks  = p_tocheader[3] - p_tocheader[2] + 1;
+  p_env->gen.i_first_track = tocheader[2];
+  p_env->gen.i_tracks  = tocheader[3] - tocheader[2] + 1;
   
   {
     int i, i_toclength;
     unsigned char *p_fulltoc;
     
-    i_toclength = 4 /* header */ + p_tocheader[0] +
-      ((unsigned int)p_tocheader[1] << 8);
+    i_toclength = 4 /* header */ + tocheader[0] +
+      ((unsigned int) tocheader[1] << 8);
     
     p_fulltoc = malloc( i_toclength );
     
     if( p_fulltoc == NULL ) {
       cdio_error( "out of memory" );
-      CloseHandle( hEvent );
       return false;
     }
     
-    /* Allocation length and buffer */
-    ssc.SRB_BufLen = i_toclength;
-    ssc.SRB_BufPointer  = p_fulltoc;
-    ssc.CDBByte[ 7 ] = (unsigned char) ( ssc.SRB_BufLen >>  8 ) & 0xff;
-    ssc.CDBByte[ 8 ] = (unsigned char) ( ssc.SRB_BufLen       ) & 0xff;
+    CDIO_MMC_SET_READ_LENGTH16(cdb.field, i_toclength);
+
+    i_status = run_scsi_cmd_aspi (p_env, OP_TIMEOUT_MS,
+				      scsi_mmc_get_cmd_len(cdb.field[0]), 
+				      &cdb, SCSI_MMC_DATA_READ, 
+				      i_toclength, p_fulltoc);
+    if( 0 != i_status ) {
+      p_env->gen.i_tracks = 0;
+    }
     
-    /* Initiate transfer */
-    ResetEvent( hEvent );
-    env->lpSendCommand( (void*) &ssc );
-    
-    /* If the command has still not been processed, wait until it's
-     * finished */
-    if( ssc.SRB_Status == SS_PENDING )
-      WaitForSingleObject( hEvent, INFINITE );
-    
-    /* check that the transfer went as planned */
-#if 0
-    if( ssc.SRB_Status != SS_COMP )
-      env->total_tracks = 0;
-#endif
-    
-    for( i = 0 ; i <= env->total_tracks ; i++ ) {
+    for( i = 0 ; i <= p_env->gen.i_tracks ; i++ ) {
       int i_index = 8 + 8 * i;
-      env->tocent[ i ].start_lsn = ((int)p_fulltoc[ i_index ] << 24) +
+      p_env->tocent[ i ].start_lsn = ((int)p_fulltoc[ i_index ] << 24) +
 	((int)p_fulltoc[ i_index+1 ] << 16) +
 	((int)p_fulltoc[ i_index+2 ] << 8) +
 	(int)p_fulltoc[ i_index+3 ];
-      env->tocent[ i ].Control   = (UCHAR)p_fulltoc[ 1 + 8 * i ];
+      p_env->tocent[ i ].Control   = (UCHAR)p_fulltoc[ 1 + 8 * i ];
       
       cdio_debug( "p_sectors: %i %lu", 
-		  i, (unsigned long int) env->tocent[i].start_lsn );
+		  i, (unsigned long int) p_env->tocent[i].start_lsn );
     }
     
     free( p_fulltoc );
   }
   
-  CloseHandle( hEvent );
-  env->gen.toc_init = true;
+  p_env->gen.toc_init = true;
   return true;
 }
 
@@ -606,112 +762,11 @@ wnaspi32_eject_media (void *user_data) {
 }
 #endif
 
-/*!
-  Return the the kind of drive capabilities of device.
-
- */
-cdio_drive_cap_t
-get_drive_cap_aspi (const _img_private_t *env) 
-{
-  int32_t i_drivetype = CDIO_DRIVE_CAP_CD_AUDIO | CDIO_DRIVE_CAP_UNKNOWN;
-  BYTE        buf[256] = { 0, };
-  HANDLE hEvent;
-  struct SRB_ExecSCSICmd ssc;
-
-  /* Create the transfer completion event */
-  hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
-  if( hEvent == NULL ) {
-    return CDIO_DRIVE_CAP_ERROR;
-  }
-
-  memset( &ssc, 0, sizeof( ssc ) );
-  
-  /* If device supports SCSI-3, then we can get the CD drive
-     capabilities, i.e. ability to read/write to CD-ROM/R/RW
-     or/and read/write to DVD-ROM/R/RW.   */
-  
-  ssc.SRB_Cmd      = SC_EXEC_SCSI_CMD;
-  ssc.SRB_Flags    = SRB_DIR_IN | SRB_EVENT_NOTIFY;
-  ssc.SRB_HaId     = LOBYTE( env->i_sid );
-  ssc.SRB_Target   = HIBYTE( env->i_sid );
-  ssc.SRB_Lun      = env->i_lun;
-  ssc.SRB_SenseLen = SENSE_LEN;
-
-  ssc.SRB_PostProc = (LPVOID) hEvent;
-  ssc.SRB_CDBLen   = 12;
-
-  /* Operation code */
-  CDIO_MMC_SET_COMMAND(ssc.CDBByte, CDIO_MMC_MODE_SENSE_10);
-  ssc.CDBByte[1]             = 0x0;
-  ssc.CDBByte[2]             = 0x3F;  /* try narrower 0x2a "mode-page" ? */
-  ssc.CDBByte[7]             = 0x01;
-  ssc.CDBByte[8]             = 0x00; 
-
-  /* Allocation length and buffer */
-  ssc.SRB_BufPointer  = buf;
-  ssc.SRB_BufLen = sizeof( buf );
-  
-  /* Initiate transfer */
-  ResetEvent( hEvent );
-  env->lpSendCommand( (void*) &ssc );
-  
-  /* If the command has still not been processed, wait until it's
-   * finished */
-  if( ssc.SRB_Status == SS_PENDING )
-    WaitForSingleObject( hEvent, INFINITE );
-  
-  CloseHandle( hEvent );
-
-  /* check that the transfer went as planned */
-  if( ssc.SRB_Status != SS_COMP ) {
-    return i_drivetype;
-  } else {
-    BYTE *p;
-    int lenData  = ((unsigned int)buf[0] << 8) + buf[1];
-    BYTE *pMax = buf + 256;
-
-    i_drivetype = 0;
-    /* set to first sense mask, and then walk through the masks */
-    p = buf + 8;
-    while( (p < &(buf[2+lenData])) && (p < pMax) )       {
-      BYTE which;
-      
-      which = p[0] & 0x3F;
-      switch( which )
-	{
-	case CDRAUDIOCTL:
-	case READERRREC:
-	case CDRPARMS:
-	  /* Don't handle theses yet. */
-	  break;
-	case CDRCAPS:
-	  /* Reader? */
-	  if (p[5] & 0x01) i_drivetype |= CDIO_DRIVE_CAP_CD_AUDIO;
-	  if (p[2] & 0x02) i_drivetype |= CDIO_DRIVE_CAP_CD_RW;
-	  if (p[2] & 0x08) i_drivetype |= CDIO_DRIVE_CAP_DVD;
-	  
-	  /* Writer? */
-	  if (p[3] & 0x01) i_drivetype |= CDIO_DRIVE_CAP_CD_R;
-	  if (p[3] & 0x10) i_drivetype |= CDIO_DRIVE_CAP_DVD_R;
-	  if (p[3] & 0x20) i_drivetype |= CDIO_DRIVE_CAP_DVD_RAM;
-	  
-	  if (p[6] & 0x08) i_drivetype |= CDIO_DRIVE_CAP_OPEN_TRAY;
-	  if (p[6] >> 5 != 0) 
-	    i_drivetype |= CDIO_DRIVE_CAP_CLOSE_TRAY;
-	  break;
-	default: ;
-	}
-      p += (p[1] + 2);
-    }
-  }
-  return i_drivetype;
-}
-
 /*!  
   Get format of track. 
 */
 track_format_t
-get_track_format_aspi(const _img_private_t *env, track_t track_num) 
+get_track_format_aspi(const _img_private_t *p_env, track_t track_num) 
 {
   MCI_OPEN_PARMS op;
   MCI_STATUS_PARMS st;
@@ -720,7 +775,7 @@ get_track_format_aspi(const _img_private_t *env, track_t track_num)
 
   memset( &op, 0, sizeof(MCI_OPEN_PARMS) );
   op.lpstrDeviceType = (LPCSTR)MCI_DEVTYPE_CD_AUDIO;
-  op.lpstrElementName = env->gen.source_name;
+  op.lpstrElementName = p_env->gen.source_name;
   
   /* Set the flags for the device type */
   i_flags = MCI_OPEN_TYPE | MCI_OPEN_TYPE_ID |
@@ -748,5 +803,3 @@ get_track_format_aspi(const _img_private_t *env, track_t track_num)
 }
 
 #endif /* HAVE_WIN32_CDROM */
-
-
