@@ -1,5 +1,5 @@
 /*
-    $Id: _cdio_sunos.c,v 1.40 2004/06/25 21:10:44 rocky Exp $
+    $Id: _cdio_sunos.c,v 1.74 2004/08/16 01:47:49 rocky Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002, 2003, 2004 Rocky Bernstein <rocky@panix.com>
@@ -38,7 +38,7 @@
 
 #ifdef HAVE_SOLARIS_CDROM
 
-static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.40 2004/06/25 21:10:44 rocky Exp $";
+static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.74 2004/08/16 01:47:49 rocky Exp $";
 
 #ifdef HAVE_GLOB_H
 #include <glob.h>
@@ -63,9 +63,10 @@ static const char _rcsid[] = "$Id: _cdio_sunos.c,v 1.40 2004/06/25 21:10:44 rock
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
+#include "cdtext_private.h"
 
-#define TOTAL_TRACKS    (env->tochdr.cdth_trk1)
-#define FIRST_TRACK_NUM (env->tochdr.cdth_trk0)
+/* not defined in dkio.h yet */
+#define DK_DVDRW 0x13
 
 /* reader */
 
@@ -87,12 +88,17 @@ typedef struct {
   
   access_mode_t access_mode;
 
+  /* Some of the more OS specific things. */
+  /* Entry info for each track, add 1 for leadout. */
+  struct cdrom_tocentry  tocent[CDIO_CD_MAX_TRACKS+1]; 
+
   /* Track information */
   struct cdrom_tochdr    tochdr;
-  struct cdrom_tocentry  tocent[100];    /* entry info for each track */
-
 } _img_private_t;
 
+static track_format_t get_track_format_solaris(void *p_user_data, 
+					       track_t i_track);
+
 static access_mode_t 
 str_to_access_mode_sunos(const char *psz_access_mode) 
 {
@@ -116,7 +122,7 @@ str_to_access_mode_sunos(const char *psz_access_mode)
   Initialize CD device.
  */
 static bool
-_cdio_init (_img_private_t *env)
+init_solaris (_img_private_t *env)
 {
 
   if (!cdio_generic_init(env)) return false;
@@ -126,6 +132,43 @@ _cdio_init (_img_private_t *env)
   return true;
 }
 
+/*!
+  Run a SCSI MMC command. 
+ 
+  p_user_data   internal CD structure.
+  i_timeout_ms   time in milliseconds we will wait for the command
+                to complete. 
+  i_cdb	        Size of p_cdb
+  p_cdb	        CDB bytes. 
+  e_direction	direction the transfer is to go.
+  i_buf	        Size of buffer
+  p_buf	        Buffer for data, both sending and receiving
+
+  Return 0 if no error.
+ */
+static int
+run_scsi_cmd_solaris( const void *p_user_data, unsigned int i_timeout_ms,
+		      unsigned int i_cdb, const scsi_mmc_cdb_t *p_cdb, 
+		      scsi_mmc_direction_t e_direction, 
+		      unsigned int i_buf, /*in/out*/ void *p_buf )
+{
+  const _img_private_t *p_env = p_user_data;
+  struct uscsi_cmd cgc;
+
+  memset (&cgc, 0, sizeof (struct uscsi_cmd));
+  cgc.uscsi_cdb = (caddr_t) p_cdb;
+
+  cgc.uscsi_flags = SCSI_MMC_DATA_READ == e_direction ? 
+    USCSI_READ : USCSI_WRITE;
+
+  cgc.uscsi_timeout = msecs2secs(i_timeout_ms);
+  cgc.uscsi_bufaddr = p_buf;   
+  cgc.uscsi_buflen  = i_buf;
+  cgc.uscsi_cdblen  = i_cdb;
+  
+  return ioctl(p_env->gen.fd, USCSICMD, &cgc);
+}
+
 /*!
    Reads audio sectors from CD device into data starting from lsn.
    Returns 0 if no error. 
@@ -135,7 +178,7 @@ _cdio_init (_img_private_t *env)
 */
 
 static int
-_cdio_read_audio_sectors (void *user_data, void *data, lsn_t lsn, 
+_read_audio_sectors_solaris (void *p_user_data, void *data, lsn_t lsn, 
 			  unsigned int nblocks)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
@@ -143,7 +186,7 @@ _cdio_read_audio_sectors (void *user_data, void *data, lsn_t lsn,
   msf_t _msf;
   struct cdrom_cdda cdda;
 
-  _img_private_t *env = user_data;
+  _img_private_t *env = p_user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0   = from_bcd8(_msf.m);
@@ -182,21 +225,15 @@ _cdio_read_audio_sectors (void *user_data, void *data, lsn_t lsn,
    from lsn. Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sector (void *env, void *data, lsn_t lsn, 
-			 bool b_form2)
+_read_mode1_sector_solaris (void *env, void *data, lsn_t lsn, 
+			    bool b_form2)
 {
 
-  char buf[M2RAW_SECTOR_SIZE] = { 0, };
 #if FIXED
   do something here. 
 #else
-  if (0 > cdio_generic_lseek(env, CDIO_CD_FRAMESIZE*lsn, SEEK_SET))
-    return -1;
-  if (0 > cdio_generic_read(env, buf, CDIO_CD_FRAMESIZE))
-    return -1;
-  memcpy (data, buf, b_form2 ? M2RAW_SECTOR_SIZE: CDIO_CD_FRAMESIZE);
+  return cdio_generic_read_form1_sector(env, data, lsn);
 #endif
-  return 0;
 }
 
 /*!
@@ -205,18 +242,18 @@ _cdio_read_mode1_sector (void *env, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode1_sectors (void *user_data, void *data, lsn_t lsn, 
-			  bool b_form2, unsigned int nblocks)
+_read_mode1_sectors_solaris (void *p_user_data, void *p_data, lsn_t lsn, 
+			     bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = p_user_data;
   unsigned int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode1_sector (env, 
-					    ((char *)data) + (blocksize * i),
-					    lsn + i, b_form2)) )
+    if ( (retval = _read_mode1_sector_solaris (p_env, 
+					    ((char *)p_data) + (blocksize * i),
+					       lsn + i, b_form2)) )
       return retval;
   }
   return 0;
@@ -227,8 +264,8 @@ _cdio_read_mode1_sectors (void *user_data, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn, 
-			 bool b_form2)
+_read_mode2_sector_solaris (void *p_user_data, void *p_data, lsn_t lsn, 
+			    bool b_form2)
 {
   char buf[CDIO_CD_FRAMESIZE_RAW] = { 0, };
   struct cdrom_msf *msf = (struct cdrom_msf *) &buf;
@@ -236,27 +273,27 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
   int offset = 0;
   struct cdrom_cdxa cd_read;
 
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = p_user_data;
 
   cdio_lba_to_msf (cdio_lsn_to_lba(lsn), &_msf);
   msf->cdmsf_min0 = from_bcd8(_msf.m);
   msf->cdmsf_sec0 = from_bcd8(_msf.s);
   msf->cdmsf_frame0 = from_bcd8(_msf.f);
   
-  if (env->gen.ioctls_debugged == 75)
+  if (p_env->gen.ioctls_debugged == 75)
     cdio_debug ("only displaying every 75th ioctl from now on");
   
-  if (env->gen.ioctls_debugged == 30 * 75)
+  if (p_env->gen.ioctls_debugged == 30 * 75)
     cdio_debug ("only displaying every 30*75th ioctl from now on");
   
-  if (env->gen.ioctls_debugged < 75 
-      || (env->gen.ioctls_debugged < (30 * 75)  
-	  && env->gen.ioctls_debugged % 75 == 0)
-      || env->gen.ioctls_debugged % (30 * 75) == 0)
+  if (p_env->gen.ioctls_debugged < 75 
+      || (p_env->gen.ioctls_debugged < (30 * 75)  
+	  && p_env->gen.ioctls_debugged % 75 == 0)
+      || p_env->gen.ioctls_debugged % (30 * 75) == 0)
     cdio_debug ("reading %2.2d:%2.2d:%2.2d",
 	       msf->cdmsf_min0, msf->cdmsf_sec0, msf->cdmsf_frame0);
   
-  env->gen.ioctls_debugged++;
+  p_env->gen.ioctls_debugged++;
   
   /* Using CDROMXA ioctl will actually use the same uscsi command
    * as ATAPI, except we don't need to be root
@@ -266,16 +303,16 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
   cd_read.cdxa_data = buf;
   cd_read.cdxa_length = 1;
   cd_read.cdxa_format = CDROM_XA_SECTOR_DATA;
-  if (ioctl (env->gen.fd, CDROMCDXA, &cd_read) == -1) {
+  if (ioctl (p_env->gen.fd, CDROMCDXA, &cd_read) == -1) {
     perror ("ioctl(..,CDROMCDXA,..)");
     return 1;
     /* exit (EXIT_FAILURE); */
   }
   
   if (b_form2)
-    memcpy (data, buf + (offset-CDIO_CD_SUBHEADER_SIZE), M2RAW_SECTOR_SIZE);
+    memcpy (p_data, buf + (offset-CDIO_CD_SUBHEADER_SIZE), M2RAW_SECTOR_SIZE);
   else
-    memcpy (((char *)data), buf + offset, CDIO_CD_FRAMESIZE);
+    memcpy (((char *)p_data), buf + offset, CDIO_CD_FRAMESIZE);
   
   return 0;
 }
@@ -286,18 +323,18 @@ _cdio_read_mode2_sector (void *user_data, void *data, lsn_t lsn,
    Returns 0 if no error. 
  */
 static int
-_cdio_read_mode2_sectors (void *user_data, void *data, lsn_t lsn, 
-			  bool b_form2, unsigned int nblocks)
+_read_mode2_sectors_solaris (void *p_user_data, void *data, lsn_t lsn, 
+			     bool b_form2, unsigned int nblocks)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *env = p_user_data;
   unsigned int i;
   int retval;
   unsigned int blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
 
   for (i = 0; i < nblocks; i++) {
-    if ( (retval = _cdio_read_mode2_sector (env, 
+    if ( (retval = _read_mode2_sector_solaris (env, 
 					    ((char *)data) + (blocksize * i),
-					    lsn + i, b_form2)) )
+					       lsn + i, b_form2)) )
       return retval;
   }
   return 0;
@@ -308,15 +345,15 @@ _cdio_read_mode2_sectors (void *user_data, void *data, lsn_t lsn,
    Return the size of the CD in logical block address (LBA) units.
  */
 static uint32_t 
-_cdio_stat_size (void *user_data)
+_cdio_stat_size (void *p_user_data)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *env = p_user_data;
 
   struct cdrom_tocentry tocent;
   uint32_t size;
 
-  tocent.cdte_track = CDIO_CDROM_LEADOUT_TRACK;
-  tocent.cdte_format = CDROM_LBA;
+  tocent.cdte_track  = CDIO_CDROM_LEADOUT_TRACK;
+  tocent.cdte_format = CDIO_CDROM_LBA;
   if (ioctl (env->gen.fd, CDROMREADTOCENTRY, &tocent) == -1)
     {
       perror ("ioctl(CDROMREADTOCENTRY)");
@@ -337,9 +374,9 @@ _cdio_stat_size (void *user_data)
   0 is returned if no error was found, and nonzero if there as an error.
 */
 static int
-_set_arg_solaris (void *user_data, const char key[], const char value[])
+_set_arg_solaris (void *p_user_data, const char key[], const char value[])
 {
-  _img_private_t *env = user_data;
+  _img_private_t *env = p_user_data;
 
   if (!strcmp (key, "source"))
     {
@@ -365,22 +402,26 @@ _set_arg_solaris (void *user_data, const char key[], const char value[])
   Return true if successful or false if an error.
 */
 static bool
-_cdio_read_toc (_img_private_t *env) 
+read_toc_solaris (void *p_user_data) 
 {
+  _img_private_t *p_env = p_user_data;
   int i;
 
   /* read TOC header */
-  if ( ioctl(env->gen.fd, CDROMREADTOCHDR, &env->tochdr) == -1 ) {
+  if ( ioctl(p_env->gen.fd, CDROMREADTOCHDR, &p_env->tochdr) == -1 ) {
     cdio_warn("%s: %s\n", 
             "error in ioctl CDROMREADTOCHDR", strerror(errno));
     return false;
   }
 
+  p_env->gen.i_first_track = p_env->tochdr.cdth_trk0;
+  p_env->gen.i_tracks      = p_env->tochdr.cdth_trk1;
+  
   /* read individual tracks */
-  for (i=env->tochdr.cdth_trk0; i<=env->tochdr.cdth_trk1; i++) {
-    env->tocent[i-1].cdte_track = i;
-    env->tocent[i-1].cdte_format = CDROM_MSF;
-    if ( ioctl(env->gen.fd, CDROMREADTOCENTRY, &env->tocent[i-1]) == -1 ) {
+  for (i=p_env->gen.i_first_track; i<=p_env->gen.i_tracks; i++) {
+    p_env->tocent[i-1].cdte_track = i;
+    p_env->tocent[i-1].cdte_format = CDIO_CDROM_MSF;
+    if ( ioctl(p_env->gen.fd, CDROMREADTOCENTRY, &p_env->tocent[i-1]) == -1 ) {
       cdio_warn("%s %d: %s\n",
               "error in ioctl CDROMREADTOCENTRY for track", 
               i, strerror(errno));
@@ -389,18 +430,18 @@ _cdio_read_toc (_img_private_t *env)
   }
 
   /* read the lead-out track */
-  env->tocent[env->tochdr.cdth_trk1].cdte_track = CDIO_CDROM_LEADOUT_TRACK;
-  env->tocent[env->tochdr.cdth_trk1].cdte_format = CDROM_MSF;
+  p_env->tocent[p_env->tochdr.cdth_trk1].cdte_track = CDIO_CDROM_LEADOUT_TRACK;
+  p_env->tocent[p_env->tochdr.cdth_trk1].cdte_format = CDIO_CDROM_MSF;
 
-  if (ioctl(env->gen.fd, CDROMREADTOCENTRY, 
-	    &env->tocent[env->tochdr.cdth_trk1]) == -1 ) {
+  if (ioctl(p_env->gen.fd, CDROMREADTOCENTRY, 
+	    &p_env->tocent[p_env->tochdr.cdth_trk1]) == -1 ) {
     cdio_warn("%s: %s\n", 
 	     "error in ioctl CDROMREADTOCENTRY for lead-out",
             strerror(errno));
     return false;
   }
 
-  env->gen.toc_init = true;
+  p_env->gen.toc_init = true;
   return true;
 }
 
@@ -409,9 +450,9 @@ _cdio_read_toc (_img_private_t *env)
   also free obj.
  */
 static int 
-_cdio_eject_media (void *user_data) {
+eject_media_solaris (void *p_user_data) {
 
-  _img_private_t *env = user_data;
+  _img_private_t *env = p_user_data;
   int ret;
 
   close(env->gen.fd);
@@ -448,9 +489,9 @@ _cdio_malloc_and_zero(size_t size) {
   Return the value associated with the key "arg".
 */
 static const char *
-_cdio_get_arg (void *user_data, const char key[])
+get_arg_solaris (void *p_user_data, const char key[])
 {
-  _img_private_t *env = user_data;
+  _img_private_t *env = p_user_data;
 
   if (!strcmp (key, "source")) {
     return env->gen.source_name;
@@ -495,173 +536,143 @@ cdio_get_default_device_solaris(void)
     }
     return device;
   }
+  /* Check if it could be a Solaris media*/
+  if((stat(DEFAULT_CDIO_DEVICE, &stb) == 0) && S_ISDIR(stb.st_mode)) {
+    device = _cdio_malloc_and_zero(strlen(DEFAULT_CDIO_DEVICE) + 4);
+    sprintf(device, "%s/s0", DEFAULT_CDIO_DEVICE);
+    return device;
+  }
   return strdup(DEFAULT_CDIO_DEVICE);
 }
 
-/*!
-  Return the the kind of drive capabilities of device.
-
-  Note: string is malloc'd so caller should free() then returned
-  string when done with it.
-
- */
-static cdio_drive_cap_t
-_cdio_get_drive_cap_solaris (const void *user_data)
-{
-#if 1
-  struct uscsi_cmd my_cmd;
-  int32_t i_drivetype = 0;
-  char buf[192] = { 0, };
-  unsigned char my_rq_buf[26];
-  unsigned char my_scsi_cdb[6];
-  const _img_private_t *env = user_data;
-  int rc;
-  
-  memset(&my_scsi_cdb, 0, sizeof(my_scsi_cdb));
-  memset(&my_rq_buf,   0, sizeof(my_rq_buf));
-  
-  /* Initialize my_scsi_cdb as a Mode Select(6) */
-  CDIO_MMC_SET_COMMAND(my_scsi_cdb, CDIO_MMC_MODE_SENSE);
-  my_scsi_cdb[1] = 0x0;  
-  my_scsi_cdb[2] = 0x2a; /* MODE_PAGE_CAPABILITIES*/;
-  my_scsi_cdb[3] = 0;    /* Not used */
-  my_scsi_cdb[4] = 128; 
-  my_scsi_cdb[5] = 0;    /* Not used */
-  
-  my_cmd.uscsi_flags = (USCSI_READ|USCSI_RQENABLE);  /* We're going get data */
-  my_cmd.uscsi_timeout = 15;             /* Allow 15 seconds for completion */
-  my_cmd.uscsi_cdb = my_scsi_cdb;        /* We'll be using the array above for the CDB */
-  my_cmd.uscsi_bufaddr = buf;   /* The block and page data is stored here */
-  my_cmd.uscsi_buflen = sizeof(buf); 
-  my_cmd.uscsi_cdblen = 6;               /* The CDB is 6 bytes long */
-  my_cmd.uscsi_rqlen =  24;              /* The request sense buffer (only valid on a check condition) is 26 bytes long */
-  my_cmd.uscsi_rqbuf = my_rq_buf;        /* Pointer to the request sense buffer */
-  
-  rc = ioctl(env->gen.fd, USCSICMD, &my_cmd);
-  if(rc == 0) {
-    unsigned int n=buf[3]+4;
-    /* Reader? */
-    if (buf[n+5] & 0x01) i_drivetype |= CDIO_DRIVE_CAP_CD_AUDIO;
-    if (buf[n+2] & 0x02) i_drivetype |= CDIO_DRIVE_CAP_CD_RW;
-    if (buf[n+2] & 0x08) i_drivetype |= CDIO_DRIVE_CAP_DVD;
-    
-    /* Writer? */
-    if (buf[n+3] & 0x01) i_drivetype |= CDIO_DRIVE_CAP_CD_R;
-    if (buf[n+3] & 0x10) i_drivetype |= CDIO_DRIVE_CAP_DVD_R;
-    if (buf[n+3] & 0x20) i_drivetype |= CDIO_DRIVE_CAP_DVD_RAM;
-    
-    if (buf[n+6] & 0x08) i_drivetype |= CDIO_DRIVE_CAP_OPEN_TRAY;
-    if (buf[n+6] >> 5 != 0) i_drivetype |= CDIO_DRIVE_CAP_CLOSE_TRAY;
-    
-  } else {
-    i_drivetype = CDIO_DRIVE_CAP_CD_AUDIO | CDIO_DRIVE_CAP_UNKNOWN;
-  }
-  return i_drivetype;
-
-#else
-  return CDIO_DRIVE_CAP_UNKNOWN | CDIO_DRIVE_CAP_CD_AUDIO 
-    | CDIO_DRIVE_CAP_CD_RW;
-#endif
-}
-
-/*!
-  Return the the kind of drive capabilities of device.
-
-  Note: string is malloc'd so caller should free() then returned
-  string when done with it.
-
- */
-static char *
-_cdio_get_mcn_solaris (const void *user_data)
-{
-#if 0
-  struct uscsi_cmd my_cmd;
-  char buf[192] = { 0, };
-  unsigned char my_rq_buf[32];
-  unsigned char my_scsi_cdb[6];
-  const _img_private_t *env = user_data;
-  int rc;
-  
-  memset(&my_scsi_cdb, 0, sizeof(my_scsi_cdb));
-  memset(&my_rq_buf,   0, sizeof(my_rq_buf));
-  
-  CDIO_MMC_SET_COMMAND(my_scsi_cdb, CDIO_MMC_GPCMD_READ_SUBCHANNEL);
-  my_scsi_cdb[1] = 0x0;  
-  my_scsi_cdb[2] = 0x40; 
-  my_scsi_cdb[3] = 02;    /* Give media catalog number. */
-  my_scsi_cdb[4] = 0;    /* Not used */
-  my_scsi_cdb[5] = 0;    /* Not used */
-  my_scsi_cdb[6] = 0;    /* Not used */
-  my_scsi_cdb[7] = 0;    /* Not used */
-  my_scsi_cdb[8] = 28; 
-  my_scsi_cdb[9] = 0;    /* Not used */
-  
-  my_cmd.uscsi_flags = (USCSI_READ|USCSI_RQENABLE);  /* We're going get data */
-  my_cmd.uscsi_timeout = 15;             /* Allow 15 seconds for completion */
-  my_cmd.uscsi_cdb = my_scsi_cdb;        /* We'll be using the array above for the CDB */
-  my_cmd.uscsi_bufaddr = buf;   /* The block and page data is stored here */
-  my_cmd.uscsi_buflen = sizeof(buf); 
-  my_cmd.uscsi_cdblen = 6;               /* The CDB is 6 bytes long */
-  my_cmd.uscsi_rqlen =  24;              /* The request sense buffer (only valid on a check condition) is 26 bytes long */
-  my_cmd.uscsi_rqbuf = my_rq_buf;        /* Pointer to the request sense buffer */
-  
-  rc = ioctl(env->gen.fd, USCSICMD, &my_cmd);
-  if(rc == 0) {
-    return strdup(&buf[9]);
-  }
-  return NULL;
-#else
-  return NULL;
-#endif
-}
-
-
-/*!
-  Return the number of of the first track. 
-  CDIO_INVALID_TRACK is returned on error.
+/*! 
+  Get disc type associated with cd object.
 */
-static track_t
-_cdio_get_first_track_num(void *user_data) 
+
+static discmode_t
+get_discmode_solaris (void *p_user_data)
 {
-  _img_private_t *env = user_data;
-  
-  if (!env->gen.toc_init) _cdio_read_toc (env) ;
+  _img_private_t *p_env = p_user_data;
+  track_t i_track;
+  discmode_t discmode=CDIO_DISC_MODE_NO_INFO;
+  struct dk_minfo media;
+  int ret;
 
-  return FIRST_TRACK_NUM;
-}
+  /* Get the media info */
+  if((ret = ioctl(p_env->gen.fd, DKIOCGMEDIAINFO, &media)) != 0) {
+     cdio_warn ("DKIOCGMEDIAINFO failed: %s\n", strerror(errno));
+	 return CDIO_DISC_MODE_NO_INFO;
+  }
+  switch(media.dki_media_type) {
+  case DK_CDROM:
+  case DK_CDR:
+  case DK_CDRW:
+  /* Do cdrom detection */
+  break;
+  case DK_DVDROM:	return CDIO_DISC_MODE_DVD_ROM;
+  case DK_DVDR:		discmode = CDIO_DISC_MODE_DVD_R;
+  break;
+  case DK_DVDRAM:	discmode = CDIO_DISC_MODE_DVD_RAM;
+  break;
+  case DK_DVDRW:
+  case DK_DVDRW+1:	discmode = CDIO_DISC_MODE_DVD_RW;
+  break;
+  default: /* no valid match */
+  return CDIO_DISC_MODE_NO_INFO; 
+  }
 
+  if((discmode == CDIO_DISC_MODE_DVD_RAM || 
+      discmode == CDIO_DISC_MODE_DVD_RW ||
+      discmode == CDIO_DISC_MODE_DVD_R)) {
+    /* Fallback to uscsi if we can */
+    if(geteuid() == 0)
+      return get_discmode_solaris(p_user_data);
+    return discmode;
+  }
 
-/*!
-  Return the number of tracks in the current medium.
-*/
-static track_t
-_cdio_get_num_tracks(void *user_data)
-{
-  _img_private_t *env = user_data;
-  
-  return TOTAL_TRACKS;
+  if (!p_env->gen.toc_init) 
+    read_toc_solaris (p_env);
+
+  if (!p_env->gen.toc_init) 
+    return CDIO_DISC_MODE_NO_INFO;
+
+  for (i_track = p_env->gen.i_first_track; 
+       i_track < p_env->gen.i_first_track + p_env->tochdr.cdth_trk1 ; 
+       i_track ++) {
+    track_format_t track_fmt=get_track_format_solaris(p_env, i_track);
+
+    switch(track_fmt) {
+    case TRACK_FORMAT_AUDIO:
+      switch(discmode) {
+	case CDIO_DISC_MODE_NO_INFO:
+	  discmode = CDIO_DISC_MODE_CD_DA;
+	  break;
+	case CDIO_DISC_MODE_CD_DA:
+	case CDIO_DISC_MODE_CD_MIXED: 
+	case CDIO_DISC_MODE_ERROR: 
+	  /* No change*/
+	  break;
+      default:
+	  discmode = CDIO_DISC_MODE_CD_MIXED;
+      }
+      break;
+    case TRACK_FORMAT_XA:
+      switch(discmode) {
+	case CDIO_DISC_MODE_NO_INFO:
+	  discmode = CDIO_DISC_MODE_CD_XA;
+	  break;
+	case CDIO_DISC_MODE_CD_XA:
+	case CDIO_DISC_MODE_CD_MIXED: 
+	case CDIO_DISC_MODE_ERROR: 
+	  /* No change*/
+	  break;
+      default:
+	discmode = CDIO_DISC_MODE_CD_MIXED;
+      }
+      break;
+    case TRACK_FORMAT_DATA:
+      switch(discmode) {
+	case CDIO_DISC_MODE_NO_INFO:
+	  discmode = CDIO_DISC_MODE_CD_DATA;
+	  break;
+	case CDIO_DISC_MODE_CD_DATA:
+	case CDIO_DISC_MODE_CD_MIXED: 
+	case CDIO_DISC_MODE_ERROR: 
+	  /* No change*/
+	  break;
+      default:
+	discmode = CDIO_DISC_MODE_CD_MIXED;
+      }
+      break;
+    case TRACK_FORMAT_ERROR:
+    default:
+      discmode = CDIO_DISC_MODE_ERROR;
+    }
+  }
+  return discmode;
 }
 
 /*!  
   Get format of track. 
 */
 static track_format_t
-_cdio_get_track_format(void *user_data, track_t i_track) 
+get_track_format_solaris(void *p_user_data, track_t i_track) 
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = p_user_data;
   
-  if ( (i_track > TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM)
+  if ( (i_track > p_env->gen.i_tracks+p_env->gen.i_first_track) 
+       || i_track < p_env->gen.i_first_track)
     return TRACK_FORMAT_ERROR;
 
-  i_track -= FIRST_TRACK_NUM;
+  i_track -= p_env->gen.i_first_track;
 
   /* This is pretty much copied from the "badly broken" cdrom_count_tracks
      in linux/cdrom.c.
    */
-  if (env->tocent[i_track].cdte_ctrl & CDROM_DATA_TRACK) {
-    if (env->tocent[i_track].cdte_format == CDIO_CDROM_CDI_TRACK)
+  if (p_env->tocent[i_track].cdte_ctrl & CDROM_DATA_TRACK) {
+    if (p_env->tocent[i_track].cdte_format == CDIO_CDROM_CDI_TRACK)
       return TRACK_FORMAT_CDI;
-    else if (env->tocent[i_track].cdte_format == CDIO_CDROM_XA_TRACK) 
+    else if (p_env->tocent[i_track].cdte_format == CDIO_CDROM_XA_TRACK) 
       return TRACK_FORMAT_XA;
     else
       return TRACK_FORMAT_DATA;
@@ -679,22 +690,23 @@ _cdio_get_track_format(void *user_data, track_t i_track)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-_cdio_get_track_green(void *user_data, track_t i_track) 
+_cdio_get_track_green(void *p_user_data, track_t i_track) 
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = p_user_data;
   
-  if (!env->gen.init) _cdio_init(env);
-  if (!env->gen.toc_init) _cdio_read_toc (env) ;
+  if (!p_env->gen.init) init_solaris(p_env);
+  if (!p_env->gen.toc_init) read_toc_solaris (p_env) ;
 
-  if (i_track > TOTAL_TRACKS+FIRST_TRACK_NUM || i_track < FIRST_TRACK_NUM)
+  if (i_track >= p_env->gen.i_tracks+p_env->gen.i_first_track 
+      || i_track < p_env->gen.i_first_track)
     return false;
 
-  i_track -= FIRST_TRACK_NUM;
+  i_track -= p_env->gen.i_first_track;
 
   /* FIXME: Dunno if this is the right way, but it's what 
      I was using in cd-info for a while.
    */
-  return ((env->tocent[i_track].cdte_ctrl & 2) != 0);
+  return ((p_env->tocent[i_track].cdte_ctrl & 2) != 0);
 }
 
 /*!  
@@ -707,22 +719,23 @@ _cdio_get_track_green(void *user_data, track_t i_track)
   False is returned if there is no entry.
 */
 static bool
-_cdio_get_track_msf(void *user_data, track_t i_track, msf_t *msf)
+_cdio_get_track_msf(void *p_user_data, track_t i_track, msf_t *msf)
 {
-  _img_private_t *env = user_data;
+  _img_private_t *p_env = p_user_data;
 
   if (NULL == msf) return false;
 
-  if (!env->gen.init) _cdio_init(env);
-  if (!env->gen.toc_init) _cdio_read_toc (env) ;
+  if (!p_env->gen.init) init_solaris(p_env);
+  if (!p_env->gen.toc_init) read_toc_solaris (p_env) ;
 
   if (i_track == CDIO_CDROM_LEADOUT_TRACK) 
-    i_track = TOTAL_TRACKS + FIRST_TRACK_NUM;
+    i_track = p_env->gen.i_tracks + p_env->gen.i_first_track;
 
-  if (i_track > (TOTAL_TRACKS+FIRST_TRACK_NUM) || i_track < FIRST_TRACK_NUM) {
+  if (i_track > (p_env->gen.i_tracks+p_env->gen.i_first_track) 
+      || i_track < p_env->gen.i_first_track) {
     return false;
   } else {
-    struct cdrom_tocentry *msf0 = &env->tocent[i_track-1];
+    struct cdrom_tocentry *msf0 = &p_env->tocent[i_track-1];
     msf->m = to_bcd8(msf0->cdte_addr.msf.minute);
     msf->s = to_bcd8(msf0->cdte_addr.msf.second);
     msf->f = to_bcd8(msf0->cdte_addr.msf.frame);
@@ -751,19 +764,39 @@ cdio_get_devices_solaris (void)
 #ifndef HAVE_SOLARIS_CDROM
   return NULL;
 #else
+  char volpath[256];
+  struct stat st;
   char **drives = NULL;
-  unsigned int num_files=0;
+  int num_files=0;
 #ifdef HAVE_GLOB_H
   unsigned int i;
   glob_t globbuf;
+
   globbuf.gl_offs = 0;
   glob("/vol/dev/aliases/cdrom*", GLOB_DOOFFS, NULL, &globbuf);
   for (i=0; i<globbuf.gl_pathc; i++) {
-    cdio_add_device_list(&drives, globbuf.gl_pathv[i], &num_files);
+    if(stat(globbuf.gl_pathv[i], &st) < 0)
+      continue;
+
+    /* Check if this is a directory, if so it's probably Solaris media */
+    if(S_ISDIR(st.st_mode)) {
+      sprintf(volpath, "%s/s0", globbuf.gl_pathv[i]);
+      if(stat(volpath, &st) == 0)
+        cdio_add_device_list(&drives, volpath, &num_files);
+	}else
+      cdio_add_device_list(&drives, globbuf.gl_pathv[i], &num_files);
   }
   globfree(&globbuf);
 #else
-  cdio_add_device_list(&drives, DEFAULT_CDIO_DEVICE, &num_files);
+  if(stat(DEFAULT_CDIO_DEVICE, &st) == 0) {
+    /* Check if this is a directory, if so it's probably Solaris media */
+    if(S_ISDIR(st.st_mode)) {
+      sprintf(volpath, "%s/s0", DEFAULT_CDIO_DEVICE);
+      if(stat(volpath, &st) == 0)
+        cdio_add_device_list(&drives, volpath, &num_files);
+    }else
+      cdio_add_device_list(&drives, DEFAULT_CDIO_DEVICE, &num_files);
+  }
 #endif /*HAVE_GLOB_H*/
   cdio_add_device_list(&drives, NULL, &num_files);
   return drives;
@@ -796,26 +829,30 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
   char *psz_source;
 
   cdio_funcs _funcs = {
-    .eject_media        = _cdio_eject_media,
+    .eject_media        = eject_media_solaris,
     .free               = cdio_generic_free,
-    .get_arg            = _cdio_get_arg,
-    .get_devices        = cdio_get_devices_solaris,
+    .get_arg            = get_arg_solaris,
+    .get_cdtext         = get_cdtext_generic,
     .get_default_device = cdio_get_default_device_solaris,
-    .get_drive_cap      = _cdio_get_drive_cap_solaris,
-    .get_first_track_num= _cdio_get_first_track_num,
-    .get_mcn            = _cdio_get_mcn_solaris,
-    .get_num_tracks     = _cdio_get_num_tracks,
-    .get_track_format   = _cdio_get_track_format,
+    .get_devices        = cdio_get_devices_solaris,
+    .get_discmode       = get_discmode_solaris,
+    .get_drive_cap      = scsi_mmc_get_drive_cap_generic,
+    .get_first_track_num= get_first_track_num_generic,
+    .get_mcn            = scsi_mmc_get_mcn_generic,
+    .get_num_tracks     = get_num_tracks_generic,
+    .get_track_format   = get_track_format_solaris,
     .get_track_green    = _cdio_get_track_green,
     .get_track_lba      = NULL, /* This could be implemented if need be. */
     .get_track_msf      = _cdio_get_track_msf,
     .lseek              = cdio_generic_lseek,
     .read               = cdio_generic_read,
-    .read_audio_sectors = _cdio_read_audio_sectors,
-    .read_mode1_sector  = _cdio_read_mode1_sector,
-    .read_mode1_sectors = _cdio_read_mode1_sectors,
-    .read_mode2_sector  = _cdio_read_mode2_sector,
-    .read_mode2_sectors = _cdio_read_mode2_sectors,
+    .read_audio_sectors = _read_audio_sectors_solaris,
+    .read_mode1_sector  = _read_mode1_sector_solaris,
+    .read_mode1_sectors = _read_mode1_sectors_solaris,
+    .read_mode2_sector  = _read_mode2_sector_solaris,
+    .read_mode2_sectors = _read_mode2_sectors_solaris,
+    .read_toc           = read_toc_solaris,
+    .run_scsi_mmc_cmd   = run_scsi_cmd_solaris,
     .stat_size          = _cdio_stat_size,
     .set_arg            = _set_arg_solaris
   };
@@ -825,9 +862,12 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
   _data->access_mode    = _AM_SUN_CTRL_SCSI;
   _data->gen.init       = false;
   _data->gen.fd         = -1;
+  _data->gen.toc_init   = false;
+  _data->gen.b_cdtext_init  = false;
+  _data->gen.b_cdtext_error = false;
 
   if (NULL == psz_orig_source) {
-    psz_source=cdio_get_default_device_solaris();
+    psz_source = cdio_get_default_device_solaris();
     if (NULL == psz_source) return NULL;
     _set_arg_solaris(_data, "source", psz_source);
     free(psz_source);
@@ -843,10 +883,10 @@ cdio_open_am_solaris (const char *psz_orig_source, const char *access_mode)
     }
   }
 
-  ret = cdio_new (_data, &_funcs);
+  ret = cdio_new ( (void *) _data, &_funcs );
   if (ret == NULL) return NULL;
 
-  if (_cdio_init(_data))
+  if (init_solaris(_data))
     return ret;
   else {
     cdio_generic_free (_data);
