@@ -1,5 +1,5 @@
 /*
-    $Id: cdio.c,v 1.56 2004/06/19 19:15:15 rocky Exp $
+    $Id: cdio.c,v 1.72 2004/08/29 03:45:35 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
@@ -39,7 +39,7 @@
 #include <cdio/logging.h>
 #include "cdio_private.h"
 
-static const char _rcsid[] = "$Id: cdio.c,v 1.56 2004/06/19 19:15:15 rocky Exp $";
+static const char _rcsid[] = "$Id: cdio.c,v 1.72 2004/08/29 03:45:35 rocky Exp $";
 
 
 const char *track_format2str[6] = 
@@ -47,6 +47,24 @@ const char *track_format2str[6] =
     "audio", "CD-i", "XA", "data", "PSX", "error"
   };
 
+/* Must match discmode enumeration */
+const char *discmode2str[] = {
+  "CD-DA", 
+  "CD-DATA Form 1", 
+  "CD DATA Form 2", 
+  "CD-ROM Mixed",
+  "DVD-ROM", 
+  "DVD-RAM", 
+  "DVD-R", 
+  "DVD-RW", 
+  "DVD+R",
+  "DVD+RW", 
+  "Unknown/unclassified DVD", 
+  "No information",
+  "Error in getting information"
+};
+
+
 /* The below array gives of the drivers that are currently available for 
    on a particular host. */
 
@@ -60,13 +78,28 @@ CdIo_driver_t CdIo_driver[CDIO_MAX_DRIVER] = { {0} };
 #define CDIO_DRIVER_UNINIT -1
 int CdIo_last_driver = CDIO_DRIVER_UNINIT;
 
+#ifdef HAVE_BSDI_CDROM
+const driver_id_t cdio_os_driver = DRIVER_BSDI;
+#elif  HAVE_FREEBSD_CDROM
+const driver_id_t cdio_os_driver = DRIVER_FREEBSD;
+#elif  HAVE_LINUX_CDROM
+const driver_id_t cdio_os_driver = DRIVER_LINUX;
+#elif  HAVE_DARWIN_CDROM
+const driver_id_t cdio_os_driver = DRIVER_OSX;
+#elif  HAVE_DARWIN_SOLARIS
+const driver_id_t cdio_os_driver = DRIVER_SOLARIS;
+#elif  HAVE_DARWIN_WIN32
+const driver_id_t cdio_os_driver = DRIVER_WIN32;
+#else 
+const driver_id_t cdio_os_driver = DRIVER_UNKNOWN;
+#endif
+
 static bool 
 cdio_have_false(void)
 {
   return false;
 }
 
-
 /* The below array gives all drivers that can possibly appear.
    on a particular host. */
 
@@ -271,6 +304,25 @@ cdio_get_arg (const CdIo *obj, const char key[])
   }
 }
 
+/*! 
+  Get cdtext information for a CdIo object .
+  
+  @param obj the CD object that may contain CD-TEXT information.
+  @return the CD-TEXT object or NULL if obj is NULL
+  or CD-TEXT information does not exist.
+*/
+const cdtext_t *
+cdio_get_cdtext (CdIo *obj, track_t i_track)
+{
+  if (obj == NULL) return NULL;
+  
+  if (obj->op.get_cdtext) {
+    return obj->op.get_cdtext (obj->env, i_track);
+  } else {
+    return NULL;
+  }
+}
+
 /*!
   Return a string containing the default CD device if none is specified.
   if CdIo is NULL (we haven't initialized a specific device driver), 
@@ -310,22 +362,31 @@ cdio_get_default_device (const CdIo *obj)
 char **
 cdio_get_devices (driver_id_t driver_id)
 {
-  CdIo *cdio;
+  /* Probably could get away with &driver_id below. */
+  driver_id_t driver_id_temp = driver_id; 
+  return cdio_get_devices_ret (&driver_id_temp);
+}
 
-  switch (driver_id) {
+char **
+cdio_get_devices_ret (/*in/out*/ driver_id_t *p_driver_id)
+{
+  CdIo *p_cdio;
+
+  switch (*p_driver_id) {
     /* FIXME: spit out unknown to give image drivers as well.  */
   case DRIVER_UNKNOWN:
   case DRIVER_DEVICE:
-    cdio = scan_for_driver(DRIVER_UNKNOWN, CDIO_MAX_DRIVER, NULL, NULL);
+    p_cdio = scan_for_driver(DRIVER_UNKNOWN, CDIO_MAX_DRIVER, NULL, NULL);
+    *p_driver_id = cdio_get_driver_id(p_cdio);
     break;
   default:
-    return (*CdIo_all_drivers[driver_id].get_devices)();
+    return (*CdIo_all_drivers[*p_driver_id].get_devices)();
   }
   
-  if (cdio == NULL) return NULL;
-  if (cdio->op.get_devices) {
-    char **devices = cdio->op.get_devices ();
-    cdio_destroy(cdio);
+  if (p_cdio == NULL) return NULL;
+  if (p_cdio->op.get_devices) {
+    char **devices = p_cdio->op.get_devices ();
+    cdio_destroy(p_cdio);
     return devices;
   } else {
     return NULL;
@@ -349,14 +410,26 @@ cdio_get_devices (driver_id_t driver_id)
   the value is NULL. This also means nothing was found.
 */
 char **
-cdio_get_devices_with_cap (char* search_devices[], 
+cdio_get_devices_with_cap (/*out*/ char* search_devices[], 
                            cdio_fs_anal_t capabilities, bool any)
+{
+  driver_id_t p_driver_id;
+  return cdio_get_devices_with_cap_ret (search_devices, capabilities, any,
+                                        &p_driver_id);
+}
+
+char **
+cdio_get_devices_with_cap_ret (/*out*/ char* search_devices[], 
+                               cdio_fs_anal_t capabilities, bool any,
+                               /*out*/ driver_id_t *p_driver_id)
 {
   char **drives=search_devices;
   char **drives_ret=NULL;
   int num_drives=0;
 
-  if (NULL == drives) drives=cdio_get_devices(DRIVER_DEVICE);
+  *p_driver_id = DRIVER_DEVICE;
+
+  if (NULL == drives) drives=cdio_get_devices_ret(p_driver_id);
   if (NULL == drives) return NULL;
 
   if (capabilities == CDIO_FS_MATCH_ALL) {
@@ -374,7 +447,7 @@ cdio_get_devices_with_cap (char* search_devices[],
     need_fs_ext = capabilities & ~CDIO_FS_MASK;
       
     for( ;  *d != NULL; d++ ) {
-      CdIo *cdio = cdio_open(*d, DRIVER_UNKNOWN);
+      CdIo *cdio = cdio_open(*d, *p_driver_id);
       
       if (NULL != cdio) {
         track_t first_track = cdio_get_first_track_num(cdio);
@@ -401,25 +474,43 @@ cdio_get_devices_with_cap (char* search_devices[],
   return drives_ret;
 }
 
-/*!
-  Return the the kind of drive capabilities of device.
-
-  Note: string is malloc'd so caller should free() then returned
-  string when done with it.
-
- */
-cdio_drive_cap_t
-cdio_get_drive_cap (const CdIo *cdio)
+/*! 
+  Get medium associated with cd_obj.
+*/
+discmode_t
+cdio_get_discmode (CdIo *cd_obj)
 {
-  cdio_drive_cap_t i_drivetype = 
-    CDIO_DRIVE_CAP_UNKNOWN | CDIO_DRIVE_CAP_CD_AUDIO; /* A safe guess. */
+  if (cd_obj == NULL) return CDIO_DISC_MODE_ERROR;
   
-  if (cdio && cdio->op.get_drive_cap) {
-    i_drivetype=cdio->op.get_drive_cap(cdio->env);
+  if (cd_obj->op.get_discmode) {
+    return cd_obj->op.get_discmode (cd_obj->env);
+  } else {
+    return CDIO_DISC_MODE_NO_INFO;
   }
-  return i_drivetype;
 }
 
+/*!
+  Return the the kind of drive capabilities of device.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+void
+cdio_get_drive_cap (const CdIo *p_cdio, 
+                    cdio_drive_read_cap_t  *p_read_cap,
+                    cdio_drive_write_cap_t *p_write_cap,
+                    cdio_drive_misc_cap_t  *p_misc_cap)
+{
+  /* This seems like a safe bet. */
+  *p_read_cap  = CDIO_DRIVE_CAP_UNKNOWN;
+  *p_write_cap = CDIO_DRIVE_CAP_UNKNOWN;
+  *p_misc_cap  = CDIO_DRIVE_CAP_UNKNOWN;
+  
+  if (p_cdio && p_cdio->op.get_drive_cap) {
+    p_cdio->op.get_drive_cap(p_cdio->env, p_read_cap, p_write_cap, p_misc_cap);
+  }
+}
 
 /*!
   Return the the kind of drive capabilities of device.
@@ -428,18 +519,23 @@ cdio_get_drive_cap (const CdIo *cdio)
   string when done with it.
 
  */
-cdio_drive_cap_t
-cdio_get_drive_cap_dev (const char *device) 
+void
+cdio_get_drive_cap_dev (const char *device,
+			cdio_drive_read_cap_t  *p_read_cap,
+			cdio_drive_write_cap_t *p_write_cap,
+			cdio_drive_misc_cap_t  *p_misc_cap)
 {
-  cdio_drive_cap_t i_drivetype = CDIO_DRIVE_CAP_UNKNOWN;
-  
+  /* This seems like a safe bet. */
   CdIo *cdio=scan_for_driver(CDIO_MIN_DRIVER, CDIO_MAX_DRIVER, 
                              device, NULL);
   if (cdio) {
-    i_drivetype=cdio_get_drive_cap(cdio);
+    cdio_get_drive_cap(cdio, p_read_cap, p_write_cap, p_misc_cap);
     cdio_destroy(cdio);
+  } else {
+    *p_read_cap  = CDIO_DRIVE_CAP_UNKNOWN;
+    *p_write_cap = CDIO_DRIVE_CAP_UNKNOWN;
+    *p_misc_cap  = CDIO_DRIVE_CAP_UNKNOWN;
   }
-  return i_drivetype;
 }
 
 
@@ -469,16 +565,16 @@ cdio_get_driver_id (const CdIo *cdio)
 
 
 /*!
-  Return the number of of the first track. 
+  Return the number of the first track. 
   CDIO_INVALID_TRACK is returned on error.
 */
 track_t
-cdio_get_first_track_num (const CdIo *cdio)
+cdio_get_first_track_num (const CdIo *p_cdio)
 {
-  cdio_assert (cdio != NULL);
+  if (NULL == p_cdio) return CDIO_INVALID_TRACK;
 
-  if (cdio->op.get_first_track_num) {
-    return cdio->op.get_first_track_num (cdio->env);
+  if (p_cdio->op.get_first_track_num) {
+    return p_cdio->op.get_first_track_num (p_cdio->env);
   } else {
     return CDIO_INVALID_TRACK;
   }
@@ -489,11 +585,29 @@ cdio_get_first_track_num (const CdIo *cdio)
   if CdIo is NULL (we haven't initialized a specific device driver), 
   then return NULL.
 */
-char *
-cdio_get_mcn (const CdIo *cdio) 
+bool
+cdio_get_hwinfo (const CdIo *p_cdio, cdio_hwinfo_t *hw_info) 
 {
-  if (cdio->op.get_mcn) {
-    return cdio->op.get_mcn (cdio->env);
+  if (!p_cdio) return false;
+  if (p_cdio->op.get_hwinfo) {
+    return p_cdio->op.get_hwinfo (p_cdio, hw_info);
+  } else {
+    /* Perhaps driver forgot to initialize.  We are no worse off Using
+       scsi_mmc than returning false here. */
+    return scsi_mmc_get_hwinfo(p_cdio, hw_info);
+  }
+}
+
+/*!
+  Return a string containing the name of the driver in use.
+  if CdIo is NULL (we haven't initialized a specific device driver), 
+  then return NULL.
+*/
+char *
+cdio_get_mcn (const CdIo *p_cdio) 
+{
+  if (p_cdio->op.get_mcn) {
+    return p_cdio->op.get_mcn (p_cdio->env);
   } else {
     return NULL;
   }
@@ -504,12 +618,12 @@ cdio_get_mcn (const CdIo *cdio)
   CDIO_INVALID_TRACK is returned on error.
 */
 track_t
-cdio_get_num_tracks (const CdIo *cdio)
+cdio_get_num_tracks (const CdIo *p_cdio)
 {
-  if (cdio == NULL) return CDIO_INVALID_TRACK;
+  if (p_cdio == NULL) return CDIO_INVALID_TRACK;
 
-  if (cdio->op.get_num_tracks) {
-    return cdio->op.get_num_tracks (cdio->env);
+  if (p_cdio->op.get_num_tracks) {
+    return p_cdio->op.get_num_tracks (p_cdio->env);
   } else {
     return CDIO_INVALID_TRACK;
   }
@@ -519,12 +633,12 @@ cdio_get_num_tracks (const CdIo *cdio)
   Get format of track. 
 */
 track_format_t
-cdio_get_track_format(const CdIo *cdio, track_t track_num)
+cdio_get_track_format(const CdIo *p_cdio, track_t i_track)
 {
-  cdio_assert (cdio != NULL);
+  cdio_assert (p_cdio != NULL);
 
-  if (cdio->op.get_track_format) {
-    return cdio->op.get_track_format (cdio->env, track_num);
+  if (p_cdio->op.get_track_format) {
+    return p_cdio->op.get_track_format (p_cdio->env, i_track);
   } else {
     return TRACK_FORMAT_ERROR;
   }
@@ -679,16 +793,19 @@ cdio_init(void)
 }
 
 CdIo *
-cdio_new (void *env, const cdio_funcs *funcs)
+cdio_new (generic_img_private_t *p_env, cdio_funcs *p_funcs)
 {
-  CdIo *new_cdio;
+  CdIo *p_new_cdio = _cdio_malloc (sizeof (CdIo));
 
-  new_cdio = _cdio_malloc (sizeof (CdIo));
-
-  new_cdio->env = env;
-  new_cdio->op = *funcs;
-
-  return new_cdio;
+  if (NULL == p_new_cdio) return NULL;
+  
+  p_new_cdio->env = p_env;      /* This is the private "environment" that
+                                   driver-dependent routines use. */
+  p_new_cdio->op  = *p_funcs;
+  p_env->cdio     = p_new_cdio; /* A way for the driver-dependent routines 
+                                   to access the higher-level general cdio 
+                                   object. */
+  return p_new_cdio;
 }
 
 /*!
