@@ -1,5 +1,5 @@
 /*
-    $Id: nrg.c,v 1.23 2004/06/18 22:55:24 rocky Exp $
+    $Id: nrg.c,v 1.38 2004/08/16 02:04:08 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 2001, 2003 Herbert Valerio Riedel <hvr@gnu.org>
@@ -22,9 +22,7 @@
    CD-image format residing inside a disk file (*.nrg).
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "image.h"
 
 #ifdef HAVE_STDIO_H
 #include <stdio.h>
@@ -40,39 +38,20 @@
 #endif
 
 #include <cdio/logging.h>
-#include <cdio/sector.h>
 #include <cdio/util.h>
 #include "cdio_assert.h"
 #include "bytesex.h"
 #include "ds.h"
-#include "cdio_private.h"
 #include "_cdio_stdio.h"
 #include "nrg.h"
 
-static const char _rcsid[] = "$Id: nrg.c,v 1.23 2004/06/18 22:55:24 rocky Exp $";
+static const char _rcsid[] = "$Id: nrg.c,v 1.38 2004/08/16 02:04:08 rocky Exp $";
 
 
 /* reader */
 
 #define DEFAULT_CDIO_DEVICE "image.nrg"
 
-typedef struct {
-  int            track_num;  /* Probably is index+1 */
-  msf_t          start_msf;
-  lba_t          start_lba;
-  int            start_index;
-  int            sec_count;  /* Number of sectors in track. Does not 
-				 include pregap before next entry. */
-  int            flags;      /* don't copy, 4 channel audio, pre emphasis */
-  track_format_t track_format;
-  bool           track_green;
-  uint16_t  datasize;        /* How much is in the portion we return back? */
-  long int  datastart;       /* Offset from begining that data starts */
-  uint16_t  endsize;         /* How much stuff at the end to skip over. This
-			       stuff may have error correction (EDC, or ECC).*/
-  uint16_t  blocksize;       /* total block size = start + size + end */
-} track_info_t;
-
 /* 
    Link element of track structure as a linked list.
    Possibly redundant with above track_info_t */
@@ -91,6 +70,17 @@ typedef struct {
      This must be first. */
   generic_img_private_t gen; 
   internal_position_t pos; 
+
+  /* This is common to all image drivers... */
+  char         *psz_cue_name;
+  char         *psz_mcn;         /* Media Catalog Number (5.22.3) */
+
+  track_info_t  tocent[CDIO_CD_MAX_TRACKS+1]; /* entry info for each track 
+					         add 1 for leadout. */
+  discmode_t    disc_mode;
+
+  /* Nero Specific stuff. Note: for the image_free to work, this *must*
+     be last. */
   bool          is_dao;          /* True if some of disk at once. False
 				    if some sort of track at once. */
   uint32_t      mtyp;            /* Value of MTYP (media type?) tag */
@@ -99,15 +89,11 @@ typedef struct {
   /* This is a hack because I don't really understnad NERO better. */
   bool            is_cues;
 
-  char         *mcn;             /* Media catalog number. */
-  track_info_t  tocent[100];     /* entry info for each track */
-  track_t       i_tracks;        /* number of tracks in image */
-  track_t       i_first_track;   /* track number of first track */
   CdioList     *mapping;         /* List of track information */
   uint32_t      size;
 } _img_private_t;
 
-static bool     parse_nrg (_img_private_t *env, const char *psz_nrg_name);
+static bool     parse_nrg (_img_private_t *env, const char *psz_cue_name);
 static uint32_t _stat_size_nrg (void *user_data);
 
 #include "image_common.h"
@@ -121,8 +107,8 @@ _register_mapping (_img_private_t *env, lsn_t start_lsn, uint32_t sec_count,
 		   track_format_t track_format, bool track_green,
 		   int flags)
 {
-  const int track_num=env->i_tracks;
-  track_info_t  *this_track=&(env->tocent[env->i_tracks]);
+  const int track_num=env->gen.i_tracks;
+  track_info_t  *this_track=&(env->tocent[env->gen.i_tracks]);
   _mapping_t *_map = _cdio_malloc (sizeof (_mapping_t));
 
   _map->start_lsn  = start_lsn;
@@ -197,10 +183,10 @@ _register_mapping (_img_private_t *env, lsn_t start_lsn, uint32_t sec_count,
   default:
     /*this_track->datasize=CDIO_CD_FRAMESIZE_RAW;*/
     cdio_warn ("track %d has unknown format %d",
-	       env->i_tracks, this_track->track_format);
+	       env->gen.i_tracks, this_track->track_format);
   }
   
-  env->i_tracks++;
+  env->gen.i_tracks++;
 
   cdio_debug ("start lsn: %lu sector count: %0lu -> %8ld (%08lx)", 
 	      (long unsigned int) start_lsn, 
@@ -242,7 +228,7 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
       footer_start = uint64_from_be (buf.v55.footer_ofs);
     } else {
       cdio_log (log_level, "Image not recognized as either version 5.0 or "
-		"version 5.5.x type NRG");
+		"version 5.5.x-6.x type NRG");
       return false;
     }
 
@@ -295,8 +281,8 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
 	    */
 	    
 	    env->is_cues       = true; /* HACK alert. */
-	    env->i_tracks      = 0;
-	    env->i_first_track = 1;
+	    env->gen.i_tracks      = 0;
+	    env->gen.i_first_track = 1;
 	    for (idx = 1; idx < entries-1; idx += 2) {
 	      lsn_t sec_count;
 	      int addrtype = _entries[idx].addr_ctrl / 16;
@@ -384,18 +370,21 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
 	  track_format_t track_format;
 	  int form2;
 
-	  env->mcn      = _cdio_malloc (CDIO_MCN_SIZE);
+	  /* We include an extra 0 byte so these can be used as C strings.*/
+	  env->psz_mcn    = _cdio_malloc (CDIO_MCN_SIZE+1);
 
 	  if (DAOX_ID == opcode) {
 	    _daox_array_t *_entries = (void *) chunk->data;
 	    form2         = _entries->_unknown[1];
 	    env->dtyp   = _entries->_unknown[19];
-	    memcpy(env->mcn, &(_entries->mcn), CDIO_MCN_SIZE);
+	    memcpy(env->psz_mcn, &(_entries->psz_mcn), CDIO_MCN_SIZE);
+	    env->psz_mcn[CDIO_MCN_SIZE] = '\0';
 	  } else {
 	    _daoi_array_t *_entries = (void *) chunk->data;
 	    form2         = _entries->_unknown[1];
 	    env->dtyp   = _entries->_unknown[19];
-	    memcpy(env->mcn, &(_entries->mcn), CDIO_MCN_SIZE);
+	    memcpy(env->psz_mcn, &(_entries->psz_mcn), CDIO_MCN_SIZE);
+	    env->psz_mcn[CDIO_MCN_SIZE] = '\0';
 	  }
 
 	  env->is_dao = true;
@@ -404,24 +393,32 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
 	  switch (env->dtyp) {
 	  case 0:
 	    /* Mode 1 */
-	    track_format = TRACK_FORMAT_DATA;
+	    track_format   = TRACK_FORMAT_DATA;
+	    env->disc_mode = CDIO_DISC_MODE_CD_DATA;
 	    break;
 	  case 2:
 	    /* Mode 2 form 1 */
-	    form2        = 0;
-	    track_format = TRACK_FORMAT_XA;
+	    form2          = 0;
+	    track_format   = TRACK_FORMAT_XA;
+	    env->disc_mode = CDIO_DISC_MODE_CD_XA;
 	    break;
 	  case 3:
 	    /* Mode 2 */
-	    track_format = TRACK_FORMAT_XA;
+	    track_format   = TRACK_FORMAT_XA;
+	    env->disc_mode = CDIO_DISC_MODE_CD_XA; /* ?? */
 	    break;
 	  case 0x6:
 	    /* Mode2 form mix */
-	  case 0x20:
-	    track_format = TRACK_FORMAT_XA;
+	    track_format   = TRACK_FORMAT_XA;
+	    env->disc_mode = CDIO_DISC_MODE_CD_MIXED;
+	    break;
+	  case 0x20: /* ??? Mode2 form 2, Mode2 raw?? */
+	    track_format   = TRACK_FORMAT_XA;
+	    env->disc_mode = CDIO_DISC_MODE_CD_XA; /* ??. */
 	    break;
 	  case 0x7:
-	    track_format = TRACK_FORMAT_AUDIO;
+	    track_format   = TRACK_FORMAT_AUDIO;
+	    env->disc_mode = CDIO_DISC_MODE_CD_DA;
 	    break;
 	  default:
 	    cdio_log (log_level, "Unknown track format %x\n", 
@@ -430,7 +427,8 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
 	  }
 	  if (0 == form2) {
 	    int i;
-	    for (i=0; i<env->i_tracks; i++) {
+	    for (i=0; i<env->gen.i_tracks; i++) {
+	      cdtext_init (&(env->gen.cdtext_track[i]));
 	      env->tocent[i].track_format= track_format;
 	      env->tocent[i].datastart   = 0;
 	      env->tocent[i].track_green = false;
@@ -445,7 +443,8 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
 	    }
 	  } else if (2 == form2) {
 	    int i;
-	    for (i=0; i<env->i_tracks; i++) {
+	    for (i=0; i<env->gen.i_tracks; i++) {
+	      cdtext_init (&(env->gen.cdtext_track[i]));
 	      env->tocent[i].track_green = true;
 	      env->tocent[i].track_format= track_format;
 	      env->tocent[i].datasize    = CDIO_CD_FRAMESIZE;
@@ -508,24 +507,44 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
 	    
 	    switch (track_mode) {
 	    case 0:
-	      track_format = TRACK_FORMAT_DATA;
-	      track_green  = false; /* ?? */
-	      blocksize    = CDIO_CD_FRAMESIZE;
+	      /* Mode 1 */
+	      track_format   = TRACK_FORMAT_DATA;
+	      track_green    = false; /* ?? */
+	      blocksize      = CDIO_CD_FRAMESIZE;
+	      env->disc_mode = CDIO_DISC_MODE_CD_DATA;
 	      break;
 	    case 2:
-	      track_format = TRACK_FORMAT_XA;
-	      track_green  = false; /* ?? */
-	      blocksize    = CDIO_CD_FRAMESIZE;
+	      /* Mode 2 form 1 */
+	      track_format   = TRACK_FORMAT_XA;
+	      track_green    = false; /* ?? */
+	      blocksize      = CDIO_CD_FRAMESIZE;
+	      env->disc_mode = CDIO_DISC_MODE_CD_XA;
 	      break;
 	    case 3:
-	      track_format = TRACK_FORMAT_XA;
-	      track_green  = true;
-	      blocksize    = M2RAW_SECTOR_SIZE;
+	      /* Mode 2 */
+	      track_format   = TRACK_FORMAT_XA;
+	      track_green    = true;
+	      blocksize      = M2RAW_SECTOR_SIZE;
+	      env->disc_mode = CDIO_DISC_MODE_CD_XA; /* ?? */
+	      break;
+	    case 06:
+	      /* Mode2 form mix */
+	      track_format   = TRACK_FORMAT_XA;
+	      track_green    = true;
+	      blocksize      = M2RAW_SECTOR_SIZE;
+	      env->disc_mode = CDIO_DISC_MODE_CD_MIXED;
+	      break;
+	    case 0x20: /* ??? Mode2 form 2, Mode2 raw?? */
+	      track_format   = TRACK_FORMAT_XA;
+	      track_green    = true;
+	      blocksize      = M2RAW_SECTOR_SIZE;
+	      env->disc_mode = CDIO_DISC_MODE_CD_XA; /* ??. */
 	      break;
 	    case 7:
-	      track_format = TRACK_FORMAT_AUDIO;
-	      track_green  = false;
-	      blocksize    = CDIO_CD_FRAMESIZE_RAW;
+	      track_format   = TRACK_FORMAT_AUDIO;
+	      track_green    = false;
+	      blocksize      = CDIO_CD_FRAMESIZE_RAW;
+	      env->disc_mode = CDIO_DISC_MODE_CD_DA;
 	      break;
 	    default:
 	      cdio_log (log_level, 
@@ -683,6 +702,13 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
       }
 	break;
 	
+      case CDTX_ID: { /* "CD TEXT" */
+	
+	cdio_log (log_level,
+		  "Don't know how to handle CD TEXT yet" );
+	break;
+      }
+
       default:
 	cdio_log (log_level,
 		  "unknown tag %8.8x seen", 
@@ -702,11 +728,14 @@ parse_nrg (_img_private_t *env, const char *psz_nrg_name)
   /* Don't use _stat_size_nrg since that will lead to recursion since
      we haven't fully initialized things yet.
   */
-  cdio_lsn_to_msf (env->size, &env->tocent[env->i_tracks].start_msf);
-  env->tocent[env->i_tracks].start_lba = cdio_lsn_to_lba(env->size);
-  env->tocent[env->i_tracks-1].sec_count = 
-    cdio_lsn_to_lba(env->size - env->tocent[env->i_tracks-1].start_lba);
+  cdio_lsn_to_msf (env->size, &env->tocent[env->gen.i_tracks].start_msf);
+  env->tocent[env->gen.i_tracks].start_lba = cdio_lsn_to_lba(env->size);
+  env->tocent[env->gen.i_tracks-1].sec_count = 
+    cdio_lsn_to_lba(env->size - env->tocent[env->gen.i_tracks-1].start_lba);
 
+  env->gen.b_cdtext_init  = true;
+  env->gen.b_cdtext_error = false;
+  env->gen.toc_init       = true;
   free(footer_buf);
   return true;
 }
@@ -728,6 +757,11 @@ _init_nrg (_img_private_t *env)
     return false;
   }
 
+  env->psz_mcn       = NULL;
+  env->disc_mode     = CDIO_DISC_MODE_NO_INFO;
+
+  cdtext_init (&(env->gen.cdtext));
+
   if ( !parse_nrg (env, env->gen.source_name) ) {
     cdio_warn ("image file %s is not a Nero image", 
 	       env->gen.source_name);
@@ -757,7 +791,7 @@ _lseek_nrg (void *user_data, off_t offset, int whence)
 
   unsigned int i;
 
-  for (i=0; i<env->i_tracks; i++) {
+  for (i=0; i<env->gen.i_tracks; i++) {
     track_info_t  *this_track=&(env->tocent[i]);
     env->pos.index = i;
     if ( (this_track->sec_count*this_track->datasize) >= offset) {
@@ -774,7 +808,7 @@ _lseek_nrg (void *user_data, off_t offset, int whence)
     env->pos.lba += this_track->sec_count;
   }
 
-  if (i==env->i_tracks) {
+  if (i==env->gen.i_tracks) {
     cdio_warn ("seeking outside range of disk image");
     return -1;
   } else
@@ -1004,8 +1038,10 @@ _free_nrg (void *user_data)
   if (NULL == env) return;
   if (NULL != env->mapping)
     _cdio_list_free (env->mapping, true); 
-  cdio_generic_stdio_free(env);
-  free(env);
+
+  /* The remaining part of the image is like the other image drivers,
+     so free that in the same way. */
+  _free_image(user_data);
 }
 
 /*!
@@ -1019,45 +1055,6 @@ _eject_media_nrg(void *obj)
   return 2;
 }
 
-/*
-  Set the device to use in I/O operations.
-*/
-static int
-_set_arg_nrg (void *user_data, const char key[], const char value[])
-{
-  _img_private_t *env = user_data;
-
-  if (!strcmp (key, "source"))
-    {
-      free (env->gen.source_name);
-
-      if (!value)
-	return -2;
-
-      env->gen.source_name = strdup (value);
-    }
-  else
-    return -1;
-
-  return 0;
-}
-
-/*!
-  Return the value associated with the key "arg".
-*/
-static const char *
-_get_arg_nrg (void *user_data, const char key[])
-{
-  _img_private_t *env = user_data;
-
-  if (!strcmp (key, "source")) {
-    return env->gen.source_name;
-  } else if (!strcmp(key, "access-mode")) {
-    return "image";
-  }
-  return NULL;
-}
-
 /*!
   Return an array of strings giving possible NRG disk images.
  */
@@ -1094,23 +1091,6 @@ cdio_get_default_device_nrg(void)
   return drive;
 }
 
-/*!
-  Return the the kind of drive capabilities of device.
-
-  Note: string is malloc'd so caller should free() then returned
-  string when done with it.
-
- */
-static cdio_drive_cap_t
-_get_drive_cap_nrg (const void *user_data) {
-
-  /* There may be more in the future but these we can handle now. 
-     Also, we know we can't handle 
-     LOCK, OPEN_TRAY, CLOSE_TRAY, SELECT_SPEED, SELECT_DISC
-  */
-  return CDIO_DRIVE_CAP_FILE | CDIO_DRIVE_CAP_MCN | CDIO_DRIVE_CAP_CD_AUDIO ;
-}
-
 /*!
   Return the number of tracks in the current medium.
   CDIO_INVALID_TRACK is returned on error.
@@ -1120,7 +1100,7 @@ _get_track_format_nrg(void *user_data, track_t track_num)
 {
   _img_private_t *env = user_data;
   
-  if (track_num > env->i_tracks || track_num == 0) 
+  if (track_num > env->gen.i_tracks || track_num == 0) 
     return TRACK_FORMAT_ERROR;
 
   if ( env->dtyp != DTYP_INVALID) {
@@ -1150,7 +1130,7 @@ _get_track_green_nrg(void *user_data, track_t track_num)
 {
   _img_private_t *env = user_data;
   
-  if (track_num > env->i_tracks || track_num == 0) 
+  if (track_num > env->gen.i_tracks || track_num == 0) 
     return false;
 
   if ( MTYP_AUDIO_CD == env->mtyp) return false;
@@ -1208,10 +1188,12 @@ cdio_open_nrg (const char *psz_source)
   cdio_funcs _funcs = {
     .eject_media        = _eject_media_nrg,
     .free               = _free_nrg,
-    .get_arg            = _get_arg_nrg,
+    .get_arg            = _get_arg_image,
+    .get_cdtext         = get_cdtext_generic,
     .get_devices        = cdio_get_devices_nrg,
     .get_default_device = cdio_get_default_device_nrg,
-    .get_drive_cap      = _get_drive_cap_nrg,
+    .get_discmode       = _get_discmode_image,
+    .get_drive_cap      = _get_drive_cap_image,
     .get_first_track_num= _get_first_track_num_image,
     .get_mcn            = _get_mcn_image,
     .get_num_tracks     = _get_num_tracks_image,
@@ -1226,37 +1208,46 @@ cdio_open_nrg (const char *psz_source)
     .read_mode1_sectors = _read_mode1_sectors_nrg,
     .read_mode2_sector  = _read_mode2_sector_nrg,
     .read_mode2_sectors = _read_mode2_sectors_nrg,
-    .set_arg            = _set_arg_nrg,
+    .set_arg            = _set_arg_image,
     .stat_size          = _stat_size_nrg,
   };
 
-  _data                 = _cdio_malloc (sizeof (_img_private_t));
-  _data->gen.init       = false;
+  _data                   = _cdio_malloc (sizeof (_img_private_t));
+  _data->gen.init         = false;
 
-  _data->i_tracks       = 0;
-  _data->mtyp           = 0; 
-  _data->dtyp           = DTYP_INVALID; 
-  _data->i_first_track  = 1;
-  _data->is_dao         = false; 
-  _data->is_cues        = false; /* FIXME: remove is_cues. */
+  _data->gen.i_tracks     = 0;
+  _data->mtyp             = 0; 
+  _data->dtyp             = DTYP_INVALID; 
+  _data->gen.i_first_track= 1;
+  _data->is_dao           = false; 
+  _data->is_cues          = false; /* FIXME: remove is_cues. */
 
-  _set_arg_nrg(_data, "source", (NULL == psz_source) 
-	       ? DEFAULT_CDIO_DEVICE: psz_source);
+  ret = cdio_new ((void *)_data, &_funcs);
 
-  ret = cdio_new (_data, &_funcs);
-
-  if (ret == NULL) return NULL;
-
-  if (!cdio_is_nrg(psz_source)) {
-    cdio_debug ("source name %s is not recognized as a NRG image", 
-		psz_source);
+  if (ret == NULL) {
+    free(_data);
     return NULL;
   }
 
+  _set_arg_image(_data, "source", (NULL == psz_source) 
+	       ? DEFAULT_CDIO_DEVICE: psz_source);
+
+  _data->psz_cue_name   = strdup(_get_arg_image(_data, "source"));
+
+  if (!cdio_is_nrg(_data->psz_cue_name)) {
+    cdio_debug ("source name %s is not recognized as a NRG image", 
+		_data->psz_cue_name);
+    _free_nrg(_data);
+    return NULL;
+  }
+
+  _set_arg_image (_data, "cue", _data->psz_cue_name);
+
   if (_init_nrg(_data))
     return ret;
   else {
     _free_nrg(_data);
+    free(ret);
     return NULL;
   }
 
