@@ -1,5 +1,5 @@
 /*
-    $Id: cd-info.c,v 1.70 2004/06/23 09:28:02 rocky Exp $
+    $Id: cd-info.c,v 1.89 2004/09/02 01:33:57 rocky Exp $
 
     Copyright (C) 2003, 2004 Rocky Bernstein <rocky@panix.com>
     Copyright (C) 1996, 1997, 1998  Gerd Knorr <kraxel@bytesex.org>
@@ -39,7 +39,9 @@
 
 #include <cdio/util.h>
 #include <cdio/cd_types.h>
+#include <cdio/cdtext.h>
 #include <cdio/iso9660.h>
+#include <cdio/scsi_mmc.h>
 
 #include "cdio_assert.h"
 #include "bytesex.h"
@@ -89,6 +91,9 @@ struct arguments
   char          *cddb_cachedir;
 #endif
   int            no_vcd;
+  int            show_dvd;
+  int            no_device;
+  int            no_disc_mode;
   uint32_t       debug_level;
   int            version_only;
   int            silent;
@@ -163,6 +168,15 @@ parse_options (int argc, const char *argv[])
      "CDDB timeout value in seconds (default 10 seconds)"},
 #endif
   
+    {"no-device-info", '\0', POPT_ARG_NONE, &opts.no_device, 0,
+     "Don't show device info, just CD info"},
+    
+    {"no-disc-mode", '\0', POPT_ARG_NONE, &opts.no_disc_mode, 0,
+     "Don't show disc-mode info"},
+    
+    {"dvd",   '\0', POPT_ARG_NONE, &opts.show_dvd, 0,
+     "Attempt to give DVD information if a DVD is found."},
+
 #ifdef HAVE_VCDINFO
     {"no-vcd",   'v', POPT_ARG_NONE, &opts.no_vcd, 0,
      "Don't look up Video CD information"},
@@ -335,23 +349,23 @@ msf_seconds(msf_t *msf)
       the number of tracks.
 */
 static unsigned long
-cddb_discid(CdIo *cdio, int num_tracks)
+cddb_discid(CdIo *p_cdio, int i_tracks)
 {
   int i,t,n=0;
   msf_t start_msf;
   msf_t msf;
   
-  for (i = 1; i <= num_tracks; i++) {
-    cdio_get_track_msf(cdio, i, &msf);
+  for (i = 1; i <= i_tracks; i++) {
+    cdio_get_track_msf(p_cdio, i, &msf);
     n += cddb_dec_digit_sum(msf_seconds(&msf));
   }
 
-  cdio_get_track_msf(cdio, 1, &start_msf);
-  cdio_get_track_msf(cdio, CDIO_CDROM_LEADOUT_TRACK, &msf);
+  cdio_get_track_msf(p_cdio, 1, &start_msf);
+  cdio_get_track_msf(p_cdio, CDIO_CDROM_LEADOUT_TRACK, &msf);
   
   t = msf_seconds(&msf) - msf_seconds(&start_msf);
   
-  return ((n % 0xff) << 24 | t << 8 | num_tracks);
+  return ((n % 0xff) << 24 | t << 8 | i_tracks);
 }
 
 
@@ -380,10 +394,38 @@ _log_handler (cdio_log_level_t level, const char message[])
   gl_default_cdio_log_handler (level, message);
 }
 
+static void 
+print_cdtext_track_info(CdIo *p_cdio, track_t i_track, const char *message) {
+  const cdtext_t *cdtext = cdio_get_cdtext(p_cdio, 0);
+
+  if (NULL != cdtext) {
+    cdtext_field_t i;
+    
+    printf("%s\n", message);
+    
+    for (i=0; i < MAX_CDTEXT_FIELDS; i++) {
+      if (cdtext->field[i]) {
+	printf("\t%s: %s\n", cdtext_field2str(i), cdtext->field[i]);
+      }
+    }
+  }
+}
+    
+static void 
+print_cdtext_info(CdIo *p_cdio, track_t i_tracks, track_t i_first_track) {
+  track_t i_last_track = i_first_track+i_tracks;
+  
+  print_cdtext_track_info(p_cdio, 0, "\nCD-TEXT for Disc:");
+  for ( ; i_first_track < i_last_track; i_first_track++ ) {
+    char msg[50];
+    sprintf(msg, "CD-TEXT for Track %d:", i_first_track);
+    print_cdtext_track_info(p_cdio, i_first_track, msg);
+  }
+}
 
 #ifdef HAVE_CDDB
 static void 
-print_cddb_info(CdIo *cdio, track_t num_tracks, track_t first_track_num) {
+print_cddb_info(CdIo *p_cdio, track_t i_tracks, track_t i_first_track) {
   int i, matches;
   
   cddb_conn_t *conn =  cddb_new();
@@ -425,14 +467,14 @@ print_cddb_info(CdIo *cdio, track_t num_tracks, track_t first_track_num) {
     fprintf(stderr, "%s: unable to create CDDB disc structure", program_name);
     goto cddb_destroy;
   }
-  for(i = 0; i < num_tracks; i++) {
+  for(i = 0; i < i_tracks; i++) {
     cddb_track_t *t = cddb_track_new(); 
-    t->frame_offset = cdio_get_track_lba(cdio, i+first_track_num);
+    t->frame_offset = cdio_get_track_lba(p_cdio, i+i_first_track);
     cddb_disc_add_track(disc, t);
   }
     
   disc->length = 
-    cdio_get_track_lba(cdio, CDIO_CDROM_LEADOUT_TRACK) 
+    cdio_get_track_lba(p_cdio, CDIO_CDROM_LEADOUT_TRACK) 
     / CDIO_CD_FRAMES_PER_SEC;
 
   if (!cddb_disc_calc_discid(disc)) {
@@ -495,7 +537,7 @@ print_vcd_info(driver_id_t driver) {
 #endif 
 
 static void
-print_iso9660_recurse (const CdIo *cdio, const char pathname[], 
+print_iso9660_recurse (const CdIo *p_cdio, const char pathname[], 
 		       cdio_fs_anal_t fs, 
 		       bool b_mode2)
 {
@@ -503,7 +545,7 @@ print_iso9660_recurse (const CdIo *cdio, const char pathname[],
   CdioList *dirlist =  _cdio_list_new ();
   CdioListNode *entnode;
 
-  entlist = iso9660_fs_readdir (cdio, pathname, b_mode2);
+  entlist = iso9660_fs_readdir (p_cdio, pathname, b_mode2);
     
   printf ("%s:\n", pathname);
 
@@ -566,7 +608,7 @@ print_iso9660_recurse (const CdIo *cdio, const char pathname[],
     {
       char *_fullname = _cdio_list_node_data (entnode);
 
-      print_iso9660_recurse (cdio, _fullname, fs, b_mode2);
+      print_iso9660_recurse (p_cdio, _fullname, fs, b_mode2);
     }
 
   _cdio_list_free (dirlist, true);
@@ -624,7 +666,7 @@ print_iso9660_fs (const CdIo *p_cdio, cdio_fs_anal_t fs,
 static void
 print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis, 
 	       cdio_fs_anal_t fs, int first_data, unsigned int num_audio, 
-	       track_t num_tracks, track_t first_track_num, 
+	       track_t i_tracks, track_t i_first_track, 
 	       track_format_t track_format, CdIo *p_cdio)
 {
   int need_lf;
@@ -633,10 +675,11 @@ print_analysis(int ms_offset, cdio_iso_analysis_t cdio_iso_analysis,
   case CDIO_FS_AUDIO:
     if (num_audio > 0) {
       printf("Audio CD, CDDB disc ID is %08lx\n", 
-	     cddb_discid(p_cdio, num_tracks));
+	     cddb_discid(p_cdio, i_tracks));
 #ifdef HAVE_CDDB
-      if (!opts.no_cddb) print_cddb_info(p_cdio, num_tracks, first_track_num);
+      if (!opts.no_cddb) print_cddb_info(p_cdio, i_tracks, i_first_track);
 #endif      
+      print_cdtext_info(p_cdio, i_tracks, i_first_track);
     }
     break;
   case CDIO_FS_ISO_9660:
@@ -773,6 +816,8 @@ init(void)
   opts.silent        = false;
   opts.list_drives   = false;
   opts.no_header     = false;
+  opts.no_device     = 0;
+  opts.no_disc_mode  = 0;
   opts.debug_level   = 0;
   opts.no_tracks     = 0;
   opts.print_iso9660 = 0;
@@ -784,7 +829,11 @@ init(void)
   opts.cddb_server   = NULL;
   opts.cddb_timeout = -1;
   opts.cddb_disable_cache = false;
-  
+#endif
+#ifdef HAVE_VCDINFO
+  opts.no_vcd        = 0;
+#else
+  opts.no_vcd        = 1;
 #endif
   opts.no_ioctl      = 0;
   opts.no_analysis   = 0;
@@ -798,20 +847,21 @@ int
 main(int argc, const char *argv[])
 {
 
-  CdIo               *cdio=NULL;
+  CdIo               *p_cdio=NULL;
   cdio_fs_anal_t      fs=CDIO_FS_AUDIO;
   int i;
   lsn_t               start_track_lsn;      /* lsn of first track */
-  lsn_t               data_start =0;        /* start of data area */
-  int                 ms_offset = 0;
-  track_t             num_tracks=0;
-  track_t             first_track_num  =  0;
-  unsigned int        num_audio   =  0;  /* # of audio tracks */
-  unsigned int        num_data    =  0;  /* # of data tracks */
-  int                 first_data  = -1;  /* # of first data track */
-  int                 first_audio = -1;  /* # of first audio track */
+  lsn_t               data_start     =  0;  /* start of data area */
+  int                 ms_offset      =  0;
+  track_t             i_tracks       =  0;
+  track_t             i_first_track  =  0;
+  unsigned int        num_audio      =  0;  /* # of audio tracks */
+  unsigned int        num_data       =  0;  /* # of data tracks */
+  int                 first_data     = -1;  /* # of first data track */
+  int                 first_audio    = -1;  /* # of first audio track */
   cdio_iso_analysis_t cdio_iso_analysis; 
   char                *media_catalog_number;  
+  discmode_t          discmode = CDIO_DISC_MODE_NO_INFO;
       
   memset(&cdio_iso_analysis, 0, sizeof(cdio_iso_analysis));
   init();
@@ -837,8 +887,8 @@ main(int argc, const char *argv[])
   switch (opts.source_image) {
   case IMAGE_UNKNOWN:
   case IMAGE_AUTO:
-    cdio = cdio_open_am (source_name, DRIVER_UNKNOWN, opts.access_mode);
-    if (cdio==NULL) {
+    p_cdio = cdio_open_am (source_name, DRIVER_UNKNOWN, opts.access_mode);
+    if (p_cdio==NULL) {
       if (source_name) {
 	err_exit("%s: Error in automatically selecting driver for input %s.\n", 
 		 program_name, source_name);
@@ -849,8 +899,8 @@ main(int argc, const char *argv[])
     } 
     break;
   case IMAGE_DEVICE:
-    cdio = cdio_open_am (source_name, DRIVER_DEVICE, opts.access_mode);
-    if (cdio==NULL) {
+    p_cdio = cdio_open_am (source_name, DRIVER_DEVICE, opts.access_mode);
+    if (p_cdio==NULL) {
       if (source_name) {
 	err_exit("%s: Error in automatically selecting CD-image driver for input %s\n", 
 	       program_name, source_name);
@@ -861,8 +911,8 @@ main(int argc, const char *argv[])
     } 
     break;
   case IMAGE_BIN:
-    cdio = cdio_open_am (source_name, DRIVER_BINCUE, opts.access_mode);
-    if (cdio==NULL) {
+    p_cdio = cdio_open_am (source_name, DRIVER_BINCUE, opts.access_mode);
+    if (p_cdio==NULL) {
       if (source_name) {
 	err_exit("%s: Error in opening bin/cue for input %s\n", 
 		 program_name, source_name);
@@ -873,8 +923,8 @@ main(int argc, const char *argv[])
     } 
     break;
   case IMAGE_CUE:
-    cdio = cdio_open_cue(source_name);
-    if (cdio==NULL) {
+    p_cdio = cdio_open_cue(source_name);
+    if (p_cdio==NULL) {
       if (source_name) {
 	err_exit("%s: Error in opening cue/bin with input %s\n", 
 		 program_name, source_name);
@@ -885,8 +935,8 @@ main(int argc, const char *argv[])
     } 
     break;
   case IMAGE_NRG:
-    cdio = cdio_open_am (source_name, DRIVER_NRG, opts.access_mode);
-    if (cdio==NULL) {
+    p_cdio = cdio_open_am (source_name, DRIVER_NRG, opts.access_mode);
+    if (p_cdio==NULL) {
       if (source_name) {
 	err_exit("%s: Error in opening NRG with input %s\n", 
 		 program_name, source_name);
@@ -898,8 +948,8 @@ main(int argc, const char *argv[])
     break;
 
   case IMAGE_CDRDAO:
-    cdio = cdio_open_am (source_name, DRIVER_CDRDAO, opts.access_mode);
-    if (cdio==NULL) {
+    p_cdio = cdio_open_am (source_name, DRIVER_CDRDAO, opts.access_mode);
+    if (p_cdio==NULL) {
       if (source_name) {
 	err_exit("%s: Error in opening TOC with input %s.\n", 
 		 program_name, source_name);
@@ -911,7 +961,7 @@ main(int argc, const char *argv[])
     break;
   }
 
-  if (cdio==NULL) {
+  if (p_cdio==NULL) {
     if (source_name) {
       err_exit("%s: Error in opening device driver for input %s.\n", 
 	       program_name, source_name);
@@ -923,53 +973,97 @@ main(int argc, const char *argv[])
   } 
 
   if (source_name==NULL) {
-    source_name=strdup(cdio_get_arg(cdio, "source"));
+    source_name=strdup(cdio_get_arg(p_cdio, "source"));
     if (NULL == source_name) {
       err_exit("%s: No input device given/found\n", program_name);
     }
   } 
 
-  if (opts.silent == 0) {
+  if (0 == opts.silent) {
     printf("CD location   : %s\n",   source_name);
-    printf("CD driver name: %s\n",   cdio_get_driver_name(cdio));
-    printf("   access mode: %s\n\n", cdio_get_arg(cdio, "access-mode"));
+    printf("CD driver name: %s\n",   cdio_get_driver_name(p_cdio));
+    printf("   access mode: %s\n\n", cdio_get_arg(p_cdio, "access-mode"));
   } 
 
-  {
-    cdio_drive_cap_t i_drive_cap =  cdio_get_drive_cap(cdio);
-    print_drive_capabilities(i_drive_cap);
+  if (0 == opts.no_device) {
+    cdio_drive_read_cap_t  i_read_cap;
+    cdio_drive_write_cap_t i_write_cap;
+    cdio_drive_misc_cap_t  i_misc_cap;
+    cdio_hwinfo_t          hwinfo;
+    if (cdio_get_hwinfo(p_cdio, &hwinfo)) {
+      printf("%-28s: %s\n%-28s: %s\n%-28s: %s\n",
+	     "Vendor"  , hwinfo.psz_vendor, 
+	     "Model"   , hwinfo.psz_model, 
+	     "Revision", hwinfo.psz_revision);
+    }
+    cdio_get_drive_cap(p_cdio, &i_read_cap, &i_write_cap, &i_misc_cap);
+    print_drive_capabilities(i_read_cap, i_write_cap, i_misc_cap);
   }
 
   if (opts.list_drives) {
-    char ** device_list = cdio_get_devices(DRIVER_DEVICE);
+    driver_id_t driver_id = DRIVER_DEVICE;
+    char ** device_list = cdio_get_devices_ret(&driver_id);
     char ** d = device_list;
 
     printf("list of devices found:\n");
     if (NULL != d) {
       for ( ; *d != NULL ; d++ ) {
-	printf("%s\n", *d);
+	CdIo *p_cdio = cdio_open(source_name, driver_id); 
+	cdio_hwinfo_t hwinfo;
+	printf("Drive %s\n", *d);
+	if (scsi_mmc_get_hwinfo(p_cdio, &hwinfo)) {
+	  printf("%-8s: %s\n%-8s: %s\n%-8s: %s\n",
+		 "Vendor"  , hwinfo.psz_vendor, 
+		 "Model"   , hwinfo.psz_model, 
+		 "Revision", hwinfo.psz_revision);
+	}
+	if (p_cdio) cdio_destroy(p_cdio);
       }
     }
     cdio_free_device_list(device_list);
     if (device_list) free(device_list);
   }
 
-  first_track_num = cdio_get_first_track_num(cdio);
-  num_tracks      = cdio_get_num_tracks(cdio);
+  printf(STRONG "\n");
+  
+
+  discmode = cdio_get_discmode(p_cdio);
+  if ( 0 == opts.no_disc_mode ) {
+    printf("Disc mode is listed as: %s\n", 
+	   discmode2str[discmode]);
+  }
+
+  if (discmode_is_dvd(discmode) && !opts.show_dvd) {
+    printf("No further information currently given for DVDs.\n");
+    printf("Use --dvd to override.\n");
+    myexit(p_cdio, EXIT_SUCCESS);
+  }
+  
+  i_first_track = cdio_get_first_track_num(p_cdio);
+
+  if (CDIO_INVALID_TRACK == i_first_track) {
+    err_exit("Can't get first track number. I give up%s.\n", "");
+  }
+
+  i_tracks = cdio_get_num_tracks(p_cdio);
+
+  if (CDIO_INVALID_TRACK == i_tracks) {
+    err_exit("Can't get number of tracks. I give up.%s\n", "");
+  }
 
   if (!opts.no_tracks) {
-    printf(STRONG "CD-ROM Track List (%i - %i)\n" NORMAL, 
-	   first_track_num, num_tracks);
+    printf("CD-ROM Track List (%i - %i)\n" NORMAL, 
+	   i_first_track, i_tracks);
 
     printf("  #: MSF       LSN     Type  Green?\n");
   }
-  
+
   /* Read and possibly print track information. */
-  for (i = first_track_num; i <= CDIO_CDROM_LEADOUT_TRACK; i++) {
+  for (i = i_first_track; i <= CDIO_CDROM_LEADOUT_TRACK; i++) {
     msf_t msf;
     char *psz_msf;
 
-    if (!cdio_get_track_msf(cdio, i, &msf)) {
+    if (!cdio_get_track_msf(p_cdio, i, &msf)) {
       err_exit("cdio_track_msf for track %i failed, I give up.\n", i);
     }
 
@@ -982,11 +1076,11 @@ main(int argc, const char *argv[])
     } else if (!opts.no_tracks) {
       printf("%3d: %8s  %06lu  %-5s %s\n", (int) i, psz_msf,
 	     (long unsigned int) cdio_msf_to_lsn(&msf),
-	     track_format2str[cdio_get_track_format(cdio, i)],
-	     cdio_get_track_green(cdio, i)? "true" : "false");
+	     track_format2str[cdio_get_track_format(p_cdio, i)],
+	     cdio_get_track_green(p_cdio, i)? "true" : "false");
     }
     
-    if (TRACK_FORMAT_AUDIO == cdio_get_track_format(cdio, i)) {
+    if (TRACK_FORMAT_AUDIO == cdio_get_track_format(p_cdio, i)) {
       num_audio++;
       if (-1 == first_audio) first_audio = i;
     } else {
@@ -994,39 +1088,28 @@ main(int argc, const char *argv[])
       if (-1 == first_data)  first_data = i;
     }
     /* skip to leadout? */
-    if (i == num_tracks) i = CDIO_CDROM_LEADOUT_TRACK-1;
+    if (i == i_tracks) i = CDIO_CDROM_LEADOUT_TRACK-1;
   }
 
-  /* get MCN */
-  media_catalog_number = cdio_get_mcn(cdio);
-  printf("Media Catalog Number (MCN): "); fflush(stdout);
-  if (NULL == media_catalog_number)
-    printf("not available\n");
-  else {
+  if (cdio_is_discmode_cdrom(discmode)) {
+    /* get and print MCN */
+    media_catalog_number = cdio_get_mcn(p_cdio);
+  
+    printf("Media Catalog Number (MCN): "); fflush(stdout);
+    if (NULL == media_catalog_number)
+      printf("not available\n");
+    else {
     printf("%s\n", media_catalog_number);
     free(media_catalog_number);
+    }
   }
   
+  
     
 #if CDIO_IOCTL_FINISHED
   if (!opts.no_ioctl) {
     printf(STRONG "What ioctl's report...\n" NORMAL);
   
-#ifdef CDROM_DISC_STATUS
-    /* get disk status */
-    printf("disc status : "); fflush(stdout);
-    switch (ioctl(filehandle,CDROM_DISC_STATUS,0)) {
-    case CDS_NO_INFO: printf("no info\n"); break;
-    case CDS_NO_DISC: printf("no disc\n"); break;
-    case CDS_AUDIO:   printf("audio\n"); break;
-    case CDS_DATA_1:  printf("data mode 1\n"); break;
-    case CDS_DATA_2:  printf("data mode 2\n"); break;
-    case CDS_XA_2_1:  printf("XA mode 1\n"); break;
-    case CDS_XA_2_2:  printf("XA mode 2\n"); break;
-    default:          printf("unknown (failed?)\n");
-    }
-#endif 
-    
 #ifdef CDROMMULTISESSION
     /* get multisession */
     printf("multisession: "); fflush(stdout);
@@ -1075,12 +1158,12 @@ main(int argc, const char *argv[])
       /* no data track, may be a "real" audio CD or hidden track CD */
       
       msf_t msf;
-      cdio_get_track_msf(cdio, 1, &msf);
+      cdio_get_track_msf(p_cdio, 1, &msf);
       start_track_lsn = cdio_msf_to_lsn(&msf);
       
       /* CD-I/Ready says start_track_lsn <= 30*75 then CDDA */
       if (start_track_lsn > 100 /* 100 is just a guess */) {
-	fs = cdio_guess_cd_type(cdio, 0, 1, &cdio_iso_analysis);
+	fs = cdio_guess_cd_type(p_cdio, 0, 1, &cdio_iso_analysis);
 	if ((CDIO_FSTYPE(fs)) != CDIO_FS_UNKNOWN)
 	  fs |= CDIO_FS_ANAL_HIDDEN_TRACK;
 	else {
@@ -1091,17 +1174,17 @@ main(int argc, const char *argv[])
 	}
       }
       print_analysis(ms_offset, cdio_iso_analysis, fs, first_data, num_audio,
-		     num_tracks, first_track_num, 
-		     cdio_get_track_format(cdio, 1), cdio);
+		     i_tracks, i_first_track, 
+		     cdio_get_track_format(p_cdio, 1), p_cdio);
     } else {
       /* we have data track(s) */
       int j;
 
-      for (j = 2, i = first_data; i <= num_tracks; i++) {
+      for (j = 2, i = first_data; i <= i_tracks; i++) {
 	msf_t msf;
-	track_format_t track_format = cdio_get_track_format(cdio, i);
+	track_format_t track_format = cdio_get_track_format(p_cdio, i);
       
-	cdio_get_track_msf(cdio, i, &msf);
+	cdio_get_track_msf(p_cdio, i, &msf);
 
 	switch ( track_format ) {
 	case TRACK_FORMAT_AUDIO:
@@ -1124,7 +1207,8 @@ main(int argc, const char *argv[])
 	if (start_track_lsn < data_start + cdio_iso_analysis.isofs_size)
 	  continue;
 	
-	fs = cdio_guess_cd_type(cdio, start_track_lsn, i, &cdio_iso_analysis);
+	fs = cdio_guess_cd_type(p_cdio, start_track_lsn, i, 
+				&cdio_iso_analysis);
 
 	if (i > 1) {
 	  /* track is beyond last session -> new session found */
@@ -1138,8 +1222,8 @@ main(int argc, const char *argv[])
 	  fs |= CDIO_FS_ANAL_MULTISESSION;
 	} else {
 	  print_analysis(ms_offset, cdio_iso_analysis, fs, first_data, 
-			 num_audio, num_tracks, first_track_num, 
-			 track_format, cdio);
+			 num_audio, i_tracks, i_first_track, 
+			 track_format, p_cdio);
 	}
 
 	if ( !(CDIO_FSTYPE(fs) == CDIO_FS_ISO_9660 ||
@@ -1153,7 +1237,7 @@ main(int argc, const char *argv[])
     }
   }
 
-  myexit(cdio, EXIT_SUCCESS);
+  myexit(p_cdio, EXIT_SUCCESS);
   /* Not reached:*/
   return(EXIT_SUCCESS);
 }
