@@ -1,5 +1,5 @@
 #!/bin/sh
-#$Id: check_cue.sh.in,v 1.19 2004/05/06 12:05:50 rocky Exp $
+#$Id: check_cue.sh.in,v 1.20 2004/07/24 14:23:39 rocky Exp $
 # Tests to see that BIN/CUE and cdrdao TOC file iamge reading is correct
 # (via cd-info).
 
@@ -40,7 +40,7 @@ check_result $RC "cd-info TOC test $testnum" "cd-info $opts"
 fname=isofs-m1
 testnum='ISO 9660 mode1 CUE'
 if test -f  ${srcdir}/${fname}.bin ; then
-  opts="-q --cue-file ${srcdir}/${fname}.cue --iso9660"
+  opts="-q --no-disc-mode --cue-file ${srcdir}/${fname}.cue --iso9660"
   test_cdinfo "$opts" ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
   check_result $RC "cd-info CUE test $testnum" "$opts"
@@ -50,7 +50,7 @@ fi
 
 testnum='ISO 9660 mode1 TOC'
 if test -f  ${srcdir}/${fname}.bin ; then
-  opts="-q --toc-file ${srcdir}/${fname}.toc --iso9660"
+  opts="-q --no-disc-mode --toc-file ${srcdir}/${fname}.toc --iso9660"
   test_cdinfo "$opts" ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
   check_result $RC "cd-info TOC test $testnum" "cd-info $opts"
@@ -66,12 +66,12 @@ else
 fi
 testnum='Video CD'
 if test -f ${srcdir}/${fname}.bin ; then
-  opts="-q -c ${srcdir}/${fname}.cue --iso9660"
+  opts="-q --no-disc-mode -c ${srcdir}/${fname}.cue --iso9660"
   test_cdinfo "$opts" ${fname}.dump $right
   RC=$?
   check_result $RC "cd-info CUE test $testnum" "cd-info $opts" 
 
-  opts="-q -t ${srcdir}/${fname}.toc --iso9660"
+  opts="-q --no-disc-mode -t ${srcdir}/${fname}.toc --iso9660"
   if test -f ${srcdir}/${fname}.toc ; then
     test_cdinfo "$opts" ${fname}.dump $right
     RC=$?
@@ -86,7 +86,7 @@ fi
 fname=svcd_ogt_test_ntsc
 testnum='Super Video CD'
 if test -f ${srcdir}/${fname}.bin ; then
-  opts="-q --cue-file ${srcdir}/${fname}.cue $vcd_opt --iso9660"
+  opts="-q --no-disc-mode --cue-file ${srcdir}/${fname}.cue $vcd_opt --iso9660"
   test_cdinfo "$opts" ${fname}.dump ${srcdir}/${fname}.right
   RC=$?
   check_result $RC "cd-info CUE test $testnum" "cd-info $opts"
